* version 1.0.0  Anders Alexandersson 28apr2004
cscript "ellip with option coefs" adofile ellip
display "begin cscript ellip with option coefs on $S_DATE at $S_TIME"

set scheme sj, permanent
set memory 1m

* helper programs to check internal consistency
program define setr /* save key results into scalars */
    scalar r = r(r)
    scalar c = r(c)
    scalar mean_y = r(mean_y)
    scalar s_y = r(s_y)
    scalar mean_x = r(mean_x)
    scalar s_x = r(s_x)
end

program define checkr /* check key results */
    assert reldif( r, r(r)) < 1e-12
    assert reldif( c, r(c)) < 1e-12
    assert reldif( mean_y, r(mean_y)) < 1e-12
    assert reldif( s_y, r(s_y)) < 1e-12
    assert reldif( mean_x, r(mean_x)) < 1e-12
    assert reldif( s_x, r(s_x)) < 1e-12
end



* test error and warning messages --------------------------------------
use auto, clear
qui reg mpg weight price

// `name' found where number expected
rcof "noisily ellip weight price, coefs c(f f)" ==7

// "may not combine options coefs and means " == 198
rcof "noisily ellip weight price, means coefs" == 198

// "may not combine options coefs and by()"
rcof "noisily ellip weight price, coefs by(foreign)" == 198

// "last estimation command must be regress" == 498
qui fit mpg weight price
rcof "noisily ellip weight price, coefs" == 498

// "`e(depvar)' is not independent variable" == 198
qui reg weight price
rcof "noisily ellip weight price, coefs" == 198

qui reg mpg weight price
// "option pool() requires option coefs
rcof "noisily ellip weight price if foreign == 0, pool(3)" == 198

// "option dlopts() requires option diameter()"
rcof "noisily ellip weight price, coefs dlopts(conn(l))" == 198

// "option plopts() requires option pool()"
rcof "noisily ellip weight price, coefs plopts(conn(l))" == 198

// "option pcopts() requires option pool()"
rcof "noisily ellip weight price, coefs pcopts(conn(l))" == 198

// "option pool() requires [if exp] or [in range]" == 198
rcof "noisily ellip weight price, coefs pool(5)" == 198

// di as err `"pool(`pool') is invalid"'
// di as text "pool() must be an integer between 1 and 100"
rcof "noisily ellip weight price if foreign == 0, coefs pool(101)" == 198

// "may not combine options pool() and diameter()"
rcof "noisily ellip weight price if foreign==0, coefs diameter(3) pool(4)" == 198

// "drop _merge before using option pool()"
gen _merge = 1
rcof "noisily ellip weight price if foreign == 0, coefs pool(10)" == 110
drop _merge

// from(`from') is invalid
rcof "noisily ellip weight price, coefs from(-1)" == 125
rcof "noisily ellip weight price, coefs from(6.3)" == 125

// from(`to') is invalid
rcof "noisily ellip weight price, coefs to(-1)" == 125
rcof "noisily ellip weight price, coefs to(6.3)" == 125
rcof "noisily ellip weight price, coefs from(1) to(1)" == 198

// di as err `"evr(`evr') invalid"
// di as text "evr() must be a floating-point number " ///
//		"between 0 and 99999"
rcof "noisily ellip weight price, coefs evr(-1)" == 198

// "option total requires option by()" == 198
rcof "noisily ellip weight price, coefs total" == 198

// "option overlay requires option by()" == 198
rcof "noisily ellip weight price, coefs overlay" == 198

// di as err `"diameter(`diameter') invalid"
// di as text "diameter() must be a floating-point number " ///
//		"between -999999 and 999999"
rcof "noi ellip weight price, coefs diameter(9999999)" == 198
rcof "noi ellip weight price, coefs diameter(-9999999)" == 198

// "The correlation coefficient could not be calculated. (r==.)"
gen grp1 = 1
gen grp2 = 2
qui reg mpg grp1 grp2
rcof "noisily ellip grp1 grp2, coefs" == 459



* find correct answers with which to compare --------------------------
* ------------------------------ Example 1 (coefs) --------------------
* Montgomery and Peck (1982, 11-13, 19, 25, 28, 33, 251, 391-392);
* Montgomery, Peck, and Vining (2001, 16-19, 23, 27, 30-31, 105-107).
* The computer output (2001, 19) increases the accuracy to 4th decimal.
* Propellant age data for Figure 9.12 (1982) or Figure 3.9 (2001).
* Published as Figure 1 in STB46.
* Another testable example using default options c(f 2) and l(95) is:
* Jobson (1991, 122 and 125-127), Figure 3.13.
use figure1, clear
assert _N == 20
qui reg strength age
assert 20 == e(N)
assert 1 == e(df_m)
assert 18 == e(df_r)
assert reldif( 165.38, e(F)) < 1e-4
assert reldif( .9018, e(r2)) < 1e-4
assert reldif( 9236, e(rmse)^2) < 1e-4
assert reldif( 1527334.95, e(mss)) < 1e-4
assert reldif( 166255, e(rss)) < 1e-4
scalar r2_a = 1 - (19/18) * (1-.9018)
assert reldif( r2_a, e(r2_a)) < 1e-4
assert reldif( -37.15, _b[age]) < 1e-4
assert reldif( 2627.82, _b[_cons]) < 1e-4
assert reldif( 2.889, _se[age]) < 1e-4
assert reldif( 44.18, _se[_cons]) < 1e-4

* test the default options l(95) and c(f 2); cf. Figure 1 in STB46
ellip age, coefs ylab(minmax, format(%9.2f)) ///
	xlab(minmax, format(%9.2f)) saving(fig1a.gph, replace) /* 1a */
assert _N == 20
assert reldif( 2627.82, r(mean_x)) < 1e-4
assert reldif( 44.18, r(s_x)) < 1e-4
assert reldif( 2745.6, r(max_x)) < 1e-4
assert reldif( 2510.0, r(min_x)) < 1e-4
assert reldif( -37.15, r(mean_y)) < 1e-4
assert reldif( 2.889, r(s_y)) < 1e-4
assert reldif( -29.450, r(max_y)) < 1e-4
assert reldif( -44.857, r(min_y)) < 1e-4
assert reldif( 1958.6, r(lambda1)) < 1e-4
assert reldif( 1.9681, r(lambda2)) < 1e-4
assert reldif( -.05719, r(m1)) < 1e-4
assert reldif( 17.485, r(m2)) < 1e-4
assert reldif( -.87375, r(r)) < 1e-4
assert reldif( 7.1091, r(c)) < 1e-4
assert reldif( 3.7405, r(b)) < 1e-4
assert reldif( 118.0, r(a)) < 1e-4
setr

* test c(f) g() nograph
capture drop y95 x95
qui reg strength age, l(90)
ellip age, coefs c(f) l(90) g(y90 x90) nograph	// 

* cf. option nodraw which merely suppresses the graph display
assert _N == 400
assert y90 ~=.
assert x90 ~=.

line y90 x90

* ellip is a non-graph graphics command. plot() adds flexibility
* test plot(plot) line_options twoway_options
* example: add a conf ellipse and an msymbol in the center
 ellip age, coefs ///
 	yvarf(%4.2f) xvarf(%4.2f) ylab(minmax) xlab(minmax) ///
 	plot(line y90 x90, clpat(longdash) || ///
   		scatteri `=r(mean_y)' `=r(mean_x)', msymbol(o)) ///
   	legend(order(1 2) label(1 "95% confidence ellipse") ///
   		label(2 "90% confidence ellipse")) ///
	saving(fig1b.gph, replace)

* graph graphics command have greater flexibility:
* dropline_options, area_options 
* similar to Figure 9.12 (1982) or Figure 3.9 (2001) + Greene (2002)
gen dropy = r(mean_y)
gen dropx = r(mean_x)
twoway (line y90 x90) ///
(area y90 x90, nodropbase bfc(gs14)) /// shade
(dropline dropy dropx, vertical msymbol(i)) ///
(dropline dropx dropy, horizontal msymbol(i)), ///
xscale(alt) ylabel(-25(5)-45) xlabel(2500(100)2800) ///
ytitle("Estimated age") xtitle("Estimated constant") ///
legend(off) saving(fig1d.gph, replace) ///
 
* test -if- and -in- conditions
use figure1, clear
qui keep if _n<10
qui reg strength age
ellip age, coefs nograph          /* all data */
assert _N == 9
setr

use figure1, clear
qui reg strength age if _n<10
ellip age if _n<10, coefs nograph /* if qualifier */
checkr
qui reg strength age in 1/9
ellip age in 1/9, coefs nograph   /* in qualifier */
assert _N == 20
checkr


* --------------------------- Example 2 (coefs) -----------------------
* Theil (1971, 102, 115-116, 132-134, 180-181, 187) textile
* consumption data -- Figure 3.2 Published as Figure 2 in STB46.
* Other testable examples using c(fadj 3) and l(95) are:
* 1. Cook and Weisberg (1999, 250-255, 545),  Figure 10.5;
* 2. Johnston (1984, 190, 193-194), Figure 5.1; and
* 3. Morrison (1983, 64-65, 126-127), Figure 3.1
use figure2, clear
assert _N == 17
qui reg logtext logprice loginc, l(99)
assert 17 == e(N)
assert 2 == e(df_m)
assert 14 == e(df_r)
assert 200 < e(F)
assert reldif( .97442, e(r2)) < 1e-4
assert reldif( .0001833, e(rmse)^2) < 1e-7
assert reldif( .0972, e(mss)) < 1e-3
assert reldif( .0029, e(rss)) < 1e-3
assert reldif( .97076, e(r2_a)) < 1e-4
assert reldif( -.83, _b[logprice]) < 1e-2
assert reldif( 1.14, _b[loginc]) < 1e-2
assert reldif( 1.37, _b[_cons]) < 1e-2
assert reldif( .04, _se[logprice]) < 1e-2
assert reldif( .16, _se[loginc]) < 1e-2
assert reldif( .31, _se[_cons]) < 1e-2

ellip logprice loginc, coefs l(99) g(y99 x99) c(fadj 3) nograph
// c(fadj 3) = F(2,n-3) = default in ellip_5.ado and ellip5.ado.
assert reldif( -.9591902520865887, r(min_y)) < 1e-5
assert reldif( -.6985301439546534, r(max_y)) < 1e-5
assert reldif( .5801493057440358, r(min_x)) < 1e-5
assert reldif( 1.706215752854334, r(max_x)) < 1e-5

assert _N == 400
assert reldif( 2*6.51, r(c)) < 1e-3 /* F = 6.51 */
assert reldif( -.83, r(mean_y)) < 1e-2
assert reldif( 1.14, r(mean_x)) < 1e-2
assert reldif( .04, r(s_y)) < 1e-2
assert reldif( .16, r(s_x)) < 1e-2
assert reldif( -.222, r(r)) < 1e-2
assert reldif( 13.03, r(c)) < 1e-2
setr

// reproduce Figure 2 in STB46
qui reg logtext logprice loginc
ellip logprice loginc, coefs g(y95 x95) plot(line y99 x99) /*
    */ c(fadj 3) saving(fig2.gph, replace) ylab(minmax) xlab(minmax) /// 2
    yvarf(%7.6f) xvarf(%7.6f) 
// ->  ____ y95  ---- y99
assert reldif( -.9275937197071804, r(min_y)) < 1e-5
assert reldif( -.7301275626011277, r(max_y)) < 1e-5
assert reldif( .7166479160562802, r(min_x)) < 1e-5
assert reldif( 1.569712911535129, r(max_x)) < 1e-5

label var y95 "hello"
ellip logprice loginc, coefs plot(line y99 x99) ///
	c(fadj 3) ylab(minmax) xlab(minmax)	///
	yvarf(%7.6f) xvarf(%7.6f)		// gen() is not specified    
assert _N == 400
assert reldif( 2*3.74, r(c)) < 1e-3 /* F = 3.74 */
assert reldif( r, r(r)) < 1e-8
assert reldif( mean_y, r(mean_y)) < 1e-8
assert reldif( s_y, r(s_y)) < 1e-8
assert reldif( mean_x, r(mean_x)) < 1e-8
assert reldif( s_x, r(s_x)) < 1e-8
// -> ____ Confidence ellipse  ---- y99



* --------------------------- Example 3 (coefs) -----------------------
* Bartels (1996, 923-924, 938) Philadelphia vote fraud data --
* Figure 1. Reproduced as Figure 3 in STB46.
use figure3, clear
assert _N == 21

* assert reg for the worst "a" subset
qui reg absentee machine if subset==2, l(50)
assert reldif( 338.0, e(rmse)) < 1e-3
assert reldif( .50, e(r2_a)) < 1e-2
assert reldif( 18, e(N)) < 1e-12
assert reldif( .01394, _b[machine]) < 1e-4
assert reldif( -200.9, _b[_cons]) < 1e-3
assert reldif( .00327, _se[machine]) < 1e-4
assert reldif( 133.4, _se[_cons]) < 1e-3

* assert reg for "bp" or full dataset
qui reg absentee machine, l(50)
assert reldif( 324.8, e(rmse)) < 1e-3
assert reldif( .46, e(r2_a)) < 1e-2
assert reldif( 21, e(N)) < 1e-12
assert reldif( .01270, _b[machine]) < 1e-4
assert reldif( -125.9, _b[_cons]) < 1e-4
assert reldif( .00298, _se[machine]) < 1e-4
assert reldif( 114.3, _se[_cons]) < 1e-3

* code the individual ci that I think are right
* cf. Stata 6 uses invt()
scalar lo_y = _b[machine] - _se[machine] * invttail(e(df_r),.25)
scalar hi_y = _b[machine] + _se[machine] * invttail(e(df_r),.25)
scalar lo_x = _b[_cons] - _se[_cons] * invttail(e(df_r),.25)
scalar hi_x = _b[_cons] + _se[_cons] * invttail(e(df_r),.25)
assert reldif( .0106, lo_y) < 1e-3
assert reldif( .0147, hi_y) < 1e-3
assert reldif( -204.5, lo_x) < 1e-3
assert reldif( -47.34, hi_x) < 1e-3

* test bp ellipse with l(50) g() l2title((...))
qui ellip machine, coefs l(50) g(ybp xbp) /*
    */ l2title("(Machine Vote Margin)") nograph
assert _N == 400
assert reldif( 1.43, r(c)) < 1e-2
assert reldif( -.78, r(r)) < 1e-2
assert reldif( .01270, r(mean_y)) < 1e-4
assert reldif( .00298, r(s_y)) < 1e-4
assert reldif( -125.9, r(mean_x)) < 1e-4
assert reldif( 114.3, r(s_x)) < 1e-3

* code the joint ci that I think are right
assert reldif( .00913, r(min_y)) < 1e-4
assert reldif( .01628, r(max_y)) < 1e-4
assert reldif( -262.91, r(min_x)) < 1e-4
assert reldif( 11.112, r(max_x)) < 1e-4
setr

* replication failure: Bartels' Figure 1 graphs individual ci
assert r(min_y) < lo_y
assert r(max_y) > hi_y
assert r(min_x) < lo_x
assert r(max_x) > hi_x

* test consistency between reg and ellip
assert reldif( _b[machine], mean_y) < 1e-12
assert reldif( _se[machine], s_y) < 1e-12
assert reldif( _b[_cons], mean_x) < 1e-12
assert reldif( _se[_cons], s_x) < 1e-12

* assert reg for "b" ellipse
qui reg absentee machine if subset==1, l(50)
assert reldif( 23.2, e(rmse)) < 1e-3
assert reldif( .95, e(r2_a)) < 1e-2
assert reldif( 3, e(N)) < 1e-12
assert reldif( .00770, _b[machine]) < 1e-4
assert reldif( 153.5, _b[_cons]) < 1e-4
assert reldif( .00127, _se[machine]) < 1e-4
assert reldif( 22.5, _se[_cons]) < 1e-3

* code the joint ci that I think are right
ellip machine if subset==1, coefs l(50) nograph
assert reldif( .00550, r(min_y)) < 1e-4
assert reldif( .009898, r(max_y)) < 1e-4
assert reldif( 114.49, r(min_x)) < 1e-4
assert reldif( 192.49, r(max_x)) < 1e-4
assert reldif( 2.99, r(c)) < 1e-2
assert reldif( -.804, r(r)) < 1e-2

* test consistency between "b" reg and ellip
assert _N == 400
assert reldif(_b[machine], r(mean_y)) < 1e-12
assert reldif(_se[machine], r(s_y)) < 1e-12
assert reldif(_b[_cons], r(mean_x)) < 1e-12
assert reldif(_se[_cons], r(s_x)) < 1e-12

* may not combine options generate() and by()
rcof "noisily ellip machine, by(subset) coefs l(50) g(yb xb)" == 198 

* reproduce Figure 3 in STB46
ellip machine if subset==1, coefs l(50) ///
	ylab(0.000(0.005)0.020) xlab(-300(100)300) ///
	yscale(range(0.000 0.020)) xscale(range(-300 300)) ///
	ymtick(0.001(0.001)0.019) xmtick(-280(20)280) pool(10) ///
	l2title(Estimated Slope) ytitle((Machine Vote Margin)) ///
	xtitle(Estimated Intercept) ///
	yvarformat(%4.3f) xvarformat(%3.0f) ////
	saving(fig3.gph, replace)			/* 3 */

// pool() works with from() and to()
ellip machine if subset==1, coefs l(50) pool(10) from(1) to(5)

// cf. reproduce Figure3 without using pool(10)
// in case you want to use fractional curve outside ellip program:
// 1. b ellipse
qui reg absentee machine if subset==1, l(50)
ellip machine if subset==1, coefs l(50) gen(y_b x_b)

// 2. bp ellipse
qui reg absentee machine, l(50)
ellip machine, coefs l(50) gen(y_bp x_bp)

// 3. fractional curve
qui gen iwt = 1 if subset==1 
qui gen y = .
qui gen x = .
range iwtrange 0 1 11
forvalues i = 1/11 {
	qui replace iwt = iwtrange[`i'] if !(subset==1)	
	reg absentee machine [iw = iwt], l(50) noheader
	qui replace y = _b[machine] in `i'/`i'
	qui replace x = _b[_cons] in `i'/`i'	
	local i = `i' + 1
}
tab y
tab x
twoway (connected y x, yvarlabel("fractionally pooled curve"))

// 4. graph the result
twoway (line y_bp x_bp, yvarformat(%4.3f) xvarformat(%3.0f)) ///
(line y_b x_b, yvarformat(%4.3f)) ///
(connected y x, yvarlabel("fractionally pooled curve")), ///
	ylab(0.000(0.005)0.020) xlab(-300(100)300) ///
	yscale(range(0.000 0.020)) xscale(range(-300 300)) ///
	ymtick(0.001(0.001)0.019) xmtick(-280(20)280) ///
	l2title(Estimated Slope) ytitle((Machine Vote Margin)) ///
	xtitle(Estimated Intercept)
//	yvarformat(%4.3f) 

	
// plot() works with pool()
qui reg absentee machine if subset==1, l(50)
tempvar y x
gen `y' = .02
replace `y' = .005 in 2/2
gen `x' = -250
replace `x' = 200 in 2/2
ellip machine if subset == 1, coefs l(50) plot(line `y' `x') ///
	ylab(0.000(0.005)0.020) xlab(-300(100)300) ///
	yscale(range(0.000 0.020)) xscale(range(-300 300)) ///
	ymtick(0.001(0.001)0.019) xmtick(-280(20)280) pool(10) ///
	l2title(Estimated Slope) ytitle((Machine Vote Margin)) ///
	xtitle(Estimated Intercept) ///
	yvarformat(%4.3f) xvarformat(%3.0f)

* verify locus curve, including saved results
* default is format(%10.0g)
mat list r(IWT), format(%9.0g)
mat list r(YVAR)
mat list r(XVAR)
matrix input iwt2 = (0\ .1\ .2\ .3\ .4\ .5\ .6\ .7\ .8\ .9\ 1)
matrix input yvar2 = ( .0076996\ /*
   */ .0096834\ .0106018\ .0111892\ .0115997\ .0119032\  /*
   */ .0121369\ .0123223\ .0124731\ .0125981\ .0127035 )
matrix input xvar2 = ( 153.4906\ /*
   */ 55.06326\ .8501632\ -34.35501\ -59.08852\ -77.42296\ /*
   */ -91.55874\ -102.7905\ -111.9299\ -119.512\ -125.9036)
assert mreldif(iwt2 , r(IWT)) < 1e-4
assert mreldif(yvar2 , r(YVAR)) < 1e-4
assert mreldif(xvar2 , r(XVAR)) < 1e-4
assert _N == 400
setr

* assert that in qualifier gives same result as if qualifier
use figure3, clear
sort subset
qui reg absentee machine in 1/3, l(50)
ellip machine in 1/3, coefs l(50) nograph
assert reldif( _b[machine], r(mean_y)) < 1e-12
assert reldif( _b[_cons], r(mean_x)) < 1e-12
assert reldif( .00550, r(min_y)) < 1e-4
assert reldif( .009898, r(max_y)) < 1e-4
assert reldif( 114.49, r(min_x)) < 1e-4
assert reldif( 192.49, r(max_x)) < 1e-4
assert reldif( 2.99, r(c)) < 1e-2
assert reldif( -.804, r(r)) < 1e-2

* assert that if dataset gives same result as in qualifier
qui keep if subset==1
qui reg absentee machine, l(50)
ellip machine, coefs l(50) nograph
assert reldif( .00550, r(min_y)) < 1e-4
assert reldif( .009898, r(max_y)) < 1e-4
assert reldif( 114.49, r(min_x)) < 1e-4
assert reldif( 192.49, r(max_x)) < 1e-4
assert reldif( 2.99, r(c)) < 1e-2
assert reldif( -.804, r(r)) < 1e-2

use figure3, clear
* test options plopts() and pcopts() 
qui reg absentee machine if subset==1, l(50) // b ellipse
ellip machine if subset==1, l(50) coefs pool(10) ///
	plopts(conn(l) clpat(longdash)) ///
	pcopts(clpat(l) ms(O)) ///
	ylab(minmax) xlab(minmax)
* must use option pool() with [if exp] or [in  range]
rcof "noisily ellip machine, coefs pool(10)" == 198

// user can specify the descriptive texts in the legends
ellip machine if subset==1, l(50) coefs pool(10) ///
	plopts(conn(l) clpat(longdash)) ///
	pcopts(clpat(l) ms(O)) ///
	ylab(minmax) xlab(minmax) ///
	   	legend(order(1 2 3) ///
		label(1 "best subset") ///
   		label(2 "fractional curve") ///
		label(3 "pooled ellipse"))
		
// plopts() and pcopts()
ellip machine if subset==1, l(50) coefs pool(10) ///
	plopts(clpat(dash) clcolor(red)) /// 
	pcopts(clpat(longdash) ms(O) clcolor(blue)) ///
	clcolor(green) ylab(minmax) xlab(minmax)


* --------------------------- Replication Failure 1 (coefs) -----------
* Bartels (1996, 923-924, 938) political violence data, Figure 2
* Replication failure: Partial test script here only for completeness
use violence, clear
assert _N == 26

* assert reg for the worst "a" subset
qui reg lndeath rep lngnp lnpop ethnic if subset==2, l(50)
assert reldif( 1.38, e(rmse)) < 1e-3
assert reldif( .72, e(r2_a)) < 1e-2
assert reldif( 16, e(N)) < 1e-12
* Replication error 1: _b[_cons] should be -19.24 but is 6.37
rcof "noisily assert reldif( -19.24, _b[_cons]) < 1e-3" == 9
assert reldif( 6.3799, _b[_cons]) < 1e-3
assert reldif( 1.855, _b[lnpop]) < 1e-3
assert reldif( -1.594, _b[lngnp]) < 1e-3
assert reldif( .8359, _b[ethnic]) < 1e-3
assert reldif( -.0188, _b[rep]) < 1e-3

* Replication error 2: se(_cons) should be 6.70 but is 6.93
rcof "noisily assert reldif( 6.70, _se[_cons]) < 1e-3" == 9
assert reldif( 6.93, _se[_cons]) < 1e-3
assert reldif( .318, _se[lnpop]) < 1e-3
assert reldif( 1.008, _se[lngnp]) < 1e-3
assert reldif( 2.118, _se[ethnic]) < 1e-3
assert reldif( .351, _se[rep]) < 1e-3

* assert reg for "bp" or full dataset
qui reg lndeath rep lngnp lnpop ethnic, l(50)
assert reldif( 1.93, e(rmse)) < 1e-3
assert reldif( .63, e(r2_a)) < 1e-2
assert reldif( 26, e(N)) < 1e-12
* Replication error 3: _b[_cons] should be -.96 but is 13.06
rcof "noisily assert reldif( -.96, _b[_cons]) < 1e-3" == 9
assert reldif( 13.06, _b[_cons] ) < 1e-3
assert reldif( 1.015, _b[lnpop]) < 1e-3
assert reldif( -1.757, _b[lngnp]) < 1e-3
assert reldif( -1.598, _b[ethnic]) < 1e-3
assert reldif( -.811, _b[rep]) < 1e-3
* Replication error 4: se(_cons) should be 5.54 but is 3.25
rcof "noisily assert reldif( _se[_cons], 5.54) < 1e-3" == 9
assert reldif( 3.25, _se[_cons]) < 1e-3
assert reldif( .297, _se[lnpop]) < 1e-3
assert reldif( .432, _se[lngnp]) < 1e-3
assert reldif( 1.908, _se[ethnic]) < 1e-3
assert reldif( .342, _se[rep]) < 1e-3

ellip rep lngnp, coefs l(50) g(ybp xbp) nograph

* Replication error 5: b in Figure 2 is negative but r is positive
qui reg lndeath rep lngnp lnpop ethnic if subset ==1, l(50)
qui corr, _coef
assert abs(r(rho) - .523) < .1
ellip rep lngnp if subset==1, coefs l(50) g(yb xb) plot(line ybp xbp)



* --------------------------- Replication failure 2 (coefs) -----------
* Bartels (1996, 923-924, 938) election data, Figure 3
* In personal communication (28 Nov 1995), Larry Bartels helped to
* explain (3) in Section 2.1 on page 916. Dr. Bartels did not reply
* to my follow-up email about the replication failures.
use elect, clear
qui reg vote gnp incumb balance, l(50)
* bp and b in Figure 3 have negative tilt but r's are positive
qui corr, _coef

rcof "noisily assert r(rho) < 0" == 9

assert abs( r(rho) - .3858 ) < .1
qui reg vote gnp incumb balance if subset == 1, l(50)
qui corr, _coef
rcof "noisily assert r(rho) < 0" == 9
assert abs(r(rho) - .221) < .1


* --------------------------- Example 4 (coefs) -----------------------
* Greene (2000, 227, 239, 250, 254, 276-278) investment data,
* Figure 7.1. The 9 character variable "inflation" requires Stata 7.
use greene, clear
qui reg invest gnp time interest inflat
* gnp and time = b2 and b3 in Greene but = b1 and b2 in ellip
assert 15 == e(N)
assert 4 == e(df_m)
assert 10 == e(df_r)
assert reldif( 88.24, e(F)) < 1e-3
assert reldif( .97245, e(r2)) < 1e-4
assert reldif( .006713, e(rmse)) < 1e-5
assert reldif( .0159025, e(mss)) < 1e-6
assert reldif( .0004508, e(rss)) < 1e-6

mat b = e(b)
mat input b2 = ( .67038, -.01658, -.0023259, -.000094011, -.50907)
assert mreldif( b2, b) < 1e-4
assert reldif( .054997, _se[gnp]) < 1e-5
assert reldif( .001972, _se[time]) < 1e-5
assert reldif( .0012189, _se[interest]) < 1e-5
assert reldif( .0013475, _se[inflat]) < 1e-5
assert reldif( .05513, _se[_cons]) < 1e-4
mat V = e(V)
*        gnp        time        interest    inflat    _cons
mat input V2 = ( /*
    */  .0030247, -.0001018,  -.0000073,  -.0000023,  -.0030102 \ /*
    */ -.0001018,  .00000389, -.00000029, -.00000004,  .0001018 \ /*
    */ -.0000073, -.00000029,  .00000149, -.00000075,  .0000056 \ /*
    */ -.0000023, -.00000004, -.00000076,  .00000182, -.0000032 \ /*
    */ -.0030102,  .00010234,  .0000056,  -.0000032,   .0030391   /*
    */ )
assert mreldif( V2, V) < 1e-6

* cf. algebraic formula around B:  (b-B)' * Vhat^(-1) * (b-B) <= 2F
qui matrix accum XX = time gnp interest inflat
mat XXinv = syminv(XX[1..5,1..5])
mat C = XXinv[1..2,1..2] * e(rmse)^2
*          time       gnp
mat input C2 = ( /*
    */  .00000389, -.000102 \ /*
    */ -.000102,    .00302    /*
    */ )
assert mreldif( C2, C) < 1e-4

* Replicate values of Figure 7.1: c(fadj 5) = F(2,n-5)
ellip gnp time, coefs c(fadj 5) g(adjy adjx) nograph
assert reldif(4.1, r(c)/2) < 1e-2
assert reldif( .67038, r(mean_y)) < 1e-4
assert reldif( -.01658, r(mean_x)) < 1e-4
* no certified value for r, s_y, s_x
assert reldif( _b[gnp], r(mean_y)) < 1e-12
assert reldif( _b[time], r(mean_x)) < 1e-12

* code the joint ci that I think are right
assert reldif( .512845, r(min_y)) < 1e-4
assert reldif( .827925, r(max_y)) < 1e-4
assert reldif( -.022228, r(min_x)) < 1e-4
assert reldif( -.010932, r(max_x)) < 1e-4

* Show in graph that hypothetical c(f 5) < replicated c(fadj 5)
ellip gnp time, coefs c(f 5) plot(line adjy adjx) /*
  */ title("Inner ellipse is c(f 5), Outer ellipse is c(fadj 5)") /*
  */ saving(fig4.gph, replace)                                      /* 4 */



* --------------------------- Example 5 (coefs) -----------------------
* Weisberg (1985, 35-37, 46-47, 97-99) fuel consumption data,
* Figures 4.3 and 4.4
use weisberg, clear
* Figure 4.3 uses (F 2) = F(2,n-2)
qui reg y x2 x1
assert 48 == e(N)
assert reldif( -32.075, _b[x1]) < 1e-4
assert reldif( 12.515, _b[x2]) < 1e-4
ellip x2 x1, coefs c(f 2) g(y43 x43) nograph
* code the joint ci that I think are right
assert reldif( 7.22639, r(min_y)) < 1e-4
assert reldif( 17.8034, r(max_y)) < 1e-4
assert reldif( -62.929, r(min_x)) < 1e-4
assert reldif( -1.2207, r(max_x)) < 1e-4

* cf. matrix formula
qui matrix accum XX = x1 x2 x3 x4
mat XXinv = syminv(XX[1..5,1..5])
mat S = XXinv[1..2,1..2]
*          x1          x2
mat input S2 = ( /*
    */ .0382636, .0022158 \ /*
    */ .0022158, .0008411   /*
    */ )
assert mreldif( S2, S) < 1e-6

* add Figure 4.3 (F 2) to Figure 4.4 (Fadj 4)
qui reg y x2 x1 x3 x4
assert 48 == e(N)
assert 4 == e(df_m)
assert 43 == e(df_r)
assert reldif( 22.70, e(F)) < 1e-3
assert reldif( .6787, e(r2)) < 1e-3
assert reldif( 4396.511, e(rmse)^2) < 1e-6
assert reldif( 399316, e(mss)) < 1e-5
assert reldif( 189049.97, e(rss)) < 1e-5
assert reldif( 377.2911, _b[_cons]) < 1e-6
assert reldif( -34.79015, _b[x1]) < 1e-6
assert reldif( 13.36449, _b[x2]) < 1e-6
assert reldif( -66.58875, _b[x3]) < 1e-6
assert reldif( -2.425889, _b[x4]) < 1e-6
assert reldif( 185.5412, _se[_cons]) < 1e-6
assert reldif( 12.97020, _se[x1]) < 1e-6
assert reldif( 1.922981, _se[x2]) < 1e-6
assert reldif( 17.22175, _se[x3]) < 1e-6
assert reldif( 3.389174, _se[x4]) < 1e-6

ellip x2 x1, coefs c(fadj 4) nograph
assert reldif( 8.49268, r(min_y)) < 1e-4
assert reldif( 18.2363, r(max_y)) < 1e-4
assert reldif( -67.649, r(min_x)) < 1e-4
assert reldif( -1.93032, r(max_x)) < 1e-4

ellip x2 x1, coefs c(fadj 4) plot(line y43 x43) /*
    */ saving(fig5.gph, replace)                                    /* 5 */
assert reldif( 8.49268, r(min_y)) < 1e-4
assert reldif( 18.2363, r(max_y)) < 1e-4
assert reldif( -67.649, r(min_x)) < 1e-4
assert reldif( -1.93032, r(max_x)) < 1e-4
assert reldif( _b[x2], r(mean_y)) < 1e-12
assert reldif( _b[x1], r(mean_x)) < 1e-12



* --------------------------- Example 6 (coefs) -----------------------
* Draper and Smith (1998, 21, 46, 635-636), Solution M.
use steam, clear
qui reg y x, l(90)
scalar MSE = e(rmse)^2
assert reldif( .7926, MSE) < 1e-3
assert reldif( -.079829, _b[x]) < 1e-5
assert reldif( 13.623005, _b[_cons]) < 1e-5

* fig8.gph graphs default c option like fig1.gph but with l(90);
* cannot replicate center dot and 95% joint conf. rectangle
ellip x, coefs l(90) saving(fig6.gph, replace)               /* 6 */
assert reldif( _b[x], r(mean_y)) < 1e-12
assert reldif( _b[_cons], r(mean_x)) < 1e-12
assert reldif( 2.55, r(c)/2) < 1e-2 /* F */
assert reldif( 4.04225, MSE * r(c)) < 1e-3 /* c */
assert reldif( -.103591, r(min_y)) < 1e-4
assert reldif( -.056067, r(max_y)) < 1e-4
assert reldif( 12.3101, r(min_x)) < 1e-4
assert reldif( 14.9359, r(max_x)) < 1e-4
