* version 1.0.1  Anders Alexandersson 5jul2004 (improved example 18)
* version 1.0.0  Anders Alexandersson 28apr2004
cscript "ellip without option coefs" adofile ellip
display "begin cscript ellip without option coefs on $S_DATE at $S_TIME"

set scheme sj, permanent
set memory 160m	// for testing the npoints option

* helper programs to check internal consistency
program define setr /* save key results into scalars */
    scalar r = r(r)
    scalar c = r(c)
    scalar mean_y = r(mean_y)
    scalar s_y = r(s_y)
    scalar mean_x = r(mean_x)
    scalar s_x = r(s_x)
end

program define checkr /* check key results */
    assert reldif( r, r(r)) < 1e-12
    assert reldif( c, r(c)) < 1e-12
    assert reldif( mean_y, r(mean_y)) < 1e-12
    assert reldif( s_y, r(s_y)) < 1e-12
    assert reldif( mean_x, r(mean_x)) < 1e-12
    assert reldif( s_x, r(s_x)) < 1e-12
end


* ----------- reproduce output for the Stata Journal article -----------
* Figure 1
sysuse auto, clear
ellip mpg weight, by(foreign, total legend(off)) ///
	total tlabel(Total as a by-group) plot(scatter mpg weight)

* Figure 2
* reproduce Galton's confidence ellipse data
range t 0 `=2*_pi' 4000
gen x = 68.25 + 4 * cos(t)
gen y = 68.25 + 3 * cos(t + acos(`=(4/3)/3'))

* extract major axis
ellip y x, c(2) nograph
local m1 = r(m1) 

*  evr(.3): create inscribing lines for m1 and n2
ellip y x, c(2) evr(.3) nograph
assert reldif( 1.067779930389807, r(m1)) < 1e-5
assert reldif( -.2809567697067325, r(m2)) < 1e-5
assert reldif( 3.559266434632692, r(n)) < 1e-5
assert reldif( r(m1)/r(n), r(evr)) < 1e-5
assert reldif( r(m1)*r(m2), -r(evr)) < 1e-5

gen a05_y = r(amin_y)
replace a05_y = r(amax_y) in 2/2
gen a05_x = r(amin_x)
replace a05_x = r(amax_x) in 2/2
gen n2_y = r(n2min_y)
replace n2_y = r(amax_y) in 2/2
gen n2_x = r(n2min_x)
replace n2_x = r(amax_x) in 2/2

ellip y x, c(2) legend(off) plot( 	///
	line a05_y a05_x, clpat(solid) text(70.5 70 "m") || 	///
	line n2_y n2_x, clpat(solid) text(67 70 "n")) 		///
 	ylab(65.25 68.25 71.25) xlab(64.25 68.25 72.25, grid)	///
	ytick(65.25(1)71.25) xtick(64.25(1)72.25)		///
	ytitle("Mid-parents") xtitle("Adult children")	///
	diameter(`m1') dlopts(clpat(dot) text(70.5 70 "m"))
assert reldif( .6759268231824285, r(e2)) < 1e-5	

* ------------- end of output for the Stata Journal article ------------	



* test error and warning messages --------------------------------------
use auto, clear
ellip weight price, clpattern(dash) clcolor(red)
ellip weight price,
ellip weight price, formula(acosr)
ellip weight price, formula(theta)


// "may not combine options generate() and by()"
rcof "noisily ellip weight price, gen(y x) by(foreign)" == 198

tempvar a b
// gen() of tempvars may lengthen the dataset
// caution: _N was #, _N now is
tempvar a b
ellip weight price, gen(`a' `b') replace nograph
rcof "noisily cf _all using auto" == 9

// "max TWO arguments allowed in constant()"
rcof "noisily ellip weight price, c(t2 3 95)" == 198

// "`statname' is an invalid statname in c(statname [#])
rcof "noisily ellip weight price, c(hello 3)" == 198

// "`#' is an invalid # in c(statname [#])"
local n = _N
rcof "noisily ellip weight price, c(f `n')" == 198

// "may not combine options coefs and means " == 198
rcof "noisily ellip weight price, means coefs" == 198

// "option means (default) requires 2 variables" == 198
rcof "noisily ellip weight" == 198

// "option dlopts() requires option diameter()"
rcof "noisily ellip weight price, dlopts(conn(l))" == 198

// `"level(`level') is invalid"'
// di as text "level() must be [an integer] between 10 and 99" 
rcof "noisily ellip weight price, l(9)" == 198

// "option overlay requires option by()"
rcof "noisily ellip weight price, overlay" == 198

// "option overlay and option by(, total) may not be combined"
rcof "noisily ellip weight price, by(foreign, total) overlay" == 198

// if `level' != $S_level
// "may not specify both constant(sd) and level(`level')"
rcof "noisily ellip weight price, c(sd) l(99)" == 198

// if `level' != $S_level 
// "may not specify both constant(sd2) and level(`level')"
// rcof "noisily ellip weight price, c(sd2) l(99)" == 198

// from(`from') is invalid
rcof "noisily ellip weight price, from(-1)" == 125
rcof "noisily ellip weight price, from(6.3)" == 125

// from(`to') is invalid
rcof "noisily ellip weight price, from(-1)" == 125
rcof "noisily ellip weight price, to(6.3)" == 125
rcof "noisily ellip weight price, from(1) to(1)" == 198

// di as err `"npoints(`npoints') is invalid"'
// di as text "npoints() must be an integer between 20 and 9999"
rcof "noisily ellip weight price, npoints(10)" == 198

// di as err `"evr(`evr') invalid"
// di as text "evr() must be a floating-point number " ///
//		"between 0 and 99999"
rcof "noisily ellip weight price, evr(-1)" == 198

// "option total requires option by()" == 198
rcof "noisily ellip weight price, total" == 198

// di as err `"diameter(`diameter') invalid"
// di as text "diameter() must be a floating-point number " ///
//		"between -999999 and 999999"
rcof "noi ellip weight price, diameter(9999999)" == 198
rcof "noi ellip weight price, diameter(-9999999)" == 198

// "by() variable takes on only one value" 198
gen grp1 = 1
rcof "noisily ellip weight price, by(grp1)" == 198

// "each combination of varlist in by() must have 3 or more observations"
qui drop in 1/50 if foreign == 0
rcof "noisily ellip weight price, by(foreign)" == 2001

// "The correlation coefficient could not be calculated. (r==.)"
gen grp2 = 2
rcof "noisily ellip grp1 grp2" == 459



* test error-prone options ---------------------------------------------
use auto, clear
// foreign: numeric variable with value labels
	// How to certify that the right legend is produced?	
ellip weight price, legend(on) // -> ____ Confidence ellipse 
ellip weight price, legend(on) yvarl("boundary") // -> ____ boundary

ellip weight price, by(foreign, total) total legend(on)
	// -> ____ Confidence ellipse [repeated for each plot]

// The by() option does not allow different options to be sent to each
// plot, e.g.title "All" instead of "Total".
// Vince Wiggins provided -grc1leg- (16 June 2003), the same as
// -graph combine- except that it displays a single common legend for
// all of the combined graphs.
// cf. example 11 which has separate plot symbol for each by-group

ellip weight price, plot(scatter weight price) legend(on) 
	// -> ____ Confidence ellipse  ---- Weight (lbs.)		

// use in TSJ [G, pages 10-11, 87]?
ellip mpg weight, by(foreign) total overlay plot(scatter mpg weight) ///
	legend(label(4 "Data points")) formula(theta)
	// replace label of 4th key from "Mileage (mpg)" to "Data points")	

ellip mpg weight, by(foreign, total legend(off)) ///
	total tlabel(Total as a by-group) plot(scatter mpg weight)
	
ellip mpg weight, by(foreign, legend(off)) total  plot(scatter mpg weight)

ellip weight price, by(foreign) total overlay
	// -> ____ Domestic  ---- Foreign  .... Total [legend(on)]
ellip weight price, by(foreign, total) total ///
	tlabel(Total (as a by-group))
ellip weight price, by(foreign, total) total ///
	tlabel("Total" (as a by-group))
ellip weight price, by(foreign, rescale ///
	title("Overall title") subtitle("Overall subtitle")) ///
	title("Extra group title", suffix) ///
	subtitle("Extra group subtitle", suffix)

// rep78: numeric variable without value labels
qui drop if rep78 == 1		// 2 obs deleted
ellip weight price, by(rep78, total missing yrescale xrescale)  ///
	total xlab(minmax, format(%9.1f)) ylab(minmax, format(%9.1f))
ellip weight price, by(rep78) overlay 
ellip weight price, by(rep78) total overlay
ellip weight price, by(rep78, missing) overlay
ellip weight price, by(rep78, missing) total overlay
	// 2nd last by-group is total; last by-group is missing

// srep78: true string variable (no value label)
gen srep78 = string(rep78)
ellip weight price, by(srep78, total missing) total ///
	xlab(minmax, format(%9.1f)) ylab(minmax, format(%9.1f))
	// 1st by-group is missing
ellip weight price, by(srep78) overlay
ellip weight price, by(srep78, missing) overlay

append using auto, nolabel
qui replace srep78 = string(rep78)

qui drop if foreign==1 & srep78 =="."	// 2 obs deleted
qui drop if srep78 =="1"		// 2 obs deleted
// cf. r==0, -1 ,1  -->  circle, negative line, positive line
 	
ellip weight price, by(foreign srep78, missing total yrescale) ///
	ylab(minmax, format(%9.1f)) total
// --> ellipse total has label -Total, Total-

ellip weight price, by(foreign srep78, missing total yrescale) ///
	ylab(minmax, format(%9.1f)) total tlabel(hello)
// --> ellipse total has label -hello, hello- 


* find correct answers with which to compare --------------------------
* ------------------------- means (new to Stata 6) --------------------
* --------------------------- Example 7 (means) -----------------------

* Johnson and Wichern (2002, 226-230) college test data -- Figure 5.3
* Typo on p. 227: D. Wichern confirmed 526.59 in email on 7 Aug. 2002
* Cannot replicate -ellip x2 x1- and -ellip x3 x1- because of x1 typo
use college, clear
qui summarize x1

assert reldif( 526.59, r(mean)) < 1e-4
assert reldif( 5691.34, r(Var)) < 1e-1
qui summarize x2
assert reldif( 54.69, r(mean)) < 1e-3
assert reldif( 126.05, r(Var)) < 1e-4
qui summarize x3
assert reldif( 25.13, r(mean)) < 1e-3
assert reldif( 23.11, r(Var)) < 1e-3

* like in Example 1: yvarf() xvarf()
ellip x3 x2, c(t2 3) yvarf(%4.0f) xvarf(%4.1f) /*
  */ ylab(23(2)27) xlab(50.5(4)58.5) /*
  */ ymtick(23(1)27) xmtick(50.5(2)58.5) g(t2y t2x) /*
  */ title("fig7: yvarf() and xvarf()") saving(fig7.gph, replace)	/* 7 */
assert reldif( 54.69, r(mean_x)) < 1e-3
assert reldif( 51.22, r(min_x)) < 1e-3
assert reldif( 58.16, r(max_x)) < 1e-3
assert reldif( 25.13, r(mean_y)) < 1e-3
assert reldif( 23.65, r(min_y)) < 1e-3
assert reldif( 26.61, r(max_y)) < 1e-3
assert reldif( .095787165839366, r(c)) < 1e-5

// test that the option c(#) produces the same result
ellip x3 x2, c(.095787165839366) nograph
assert reldif( 54.69, r(mean_x)) < 1e-3
assert reldif( 51.22, r(min_x)) < 1e-3
assert reldif( 58.16, r(max_x)) < 1e-3
assert reldif( 25.13, r(mean_y)) < 1e-3
assert reldif( 23.65, r(min_y)) < 1e-3
assert reldif( 26.61, r(max_y)) < 1e-3
assert reldif( .095787165839366, r(c)) < 1e-5


* --------------------------------------------------------------------
// cf. Galton's Table 1 and Plate X data do not create the same ellipse
* Galton's Table 1 data (population data, N=928)
use galton, clear
assert _N==102
// ellip does not allow weights but expand will fweight, see [U] 23.16
qui expand freq
assert _N==928	// Table 1 data
	
// 1. parent height as function of child height
// 1A. "horizontal tangents" = evr(0), regress x on y
ellip parentht childht, c(2.5) ylab(minmax) xlab(minmax) ///
	xline(68.08847) yline(68.30819) evr(0) legend(off)	///
	plot(scatter parentht childht, mlabel(freq) m(i))	
// Galton (1886) in Family likeness, page 49: about 68.3 inches
assert reldif( r(m1), 1.547291617518644) < 1e-5
assert reldif( 68.30818965517241, r(mean_y)) < 1e-5
assert reldif( 68.08846982758621, r(mean_x)) < 1e-5 
assert reldif( 66.26203826693369, r(amin_x)) < 1e-5
assert reldif( 69.91490138823873, r(amax_x)) < 1e-5
assert reldif( 65.48216741140327, r(amin_y)) < 1e-5
assert reldif( 71.13421189894156, r(amax_y)) < 1e-5
assert reldif( 64.55092554316069, r(bmin_x)) < 1e-5
assert reldif( 71.62601411201173, r(bmax_x)) < 1e-5
assert reldif( 65.48216749626349, r(min_y)) < 1e-5
assert reldif( 71.13412966835631, r(max_y)) < 1e-5
assert reldif( 67.01175718375708, r(xmin_y)) < 1e-5
assert reldif( 69.60466231354056, r(xmax_y)) < 1e-5
assert reldif( 68.30818965517241, r(bmin_y)) < 1e-5
assert reldif( 68.30818965517241, r(bmax_y)) < 1e-5
assert reldif( 64.10737836643611, r(min_x)) < 1e-5	
assert reldif( 72.06968469368165, r(max_x)) < 1e-5
assert reldif( 1.547291617518644, r(m1)) < 1e-5
assert reldif( 0, r(m2)) < 1e-5
assert reldif( ., r(n)) < 1e-5
assert reldif( 0, r(evr)) < 1e-12
assert reldif( 4.290308657623028, r(a)) < 1e-12
assert reldif( 2.330177065079445, r(b)) < 1e-12
assert reldif( 3.364855712809339, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
rcof "noisily assert reldif( r(m1) / r(n), r(evr)) < 1e-5" == 9	
// cannot divide by 0
assert reldif( 2.5, r(c)) < 1e-5
assert reldif( .4599066315805654, r(theta)) < 1e-5
assert reldif( .7050144304027224, r(e2)) < 1e-5
assert reldif( .4587623682928201, r(r)) < 1e-5
assert reldif( 65.48216749634075, r(n2min_y)) < 1e-5	
assert reldif( 65.48216749634075, r(n1max_y)) < 1e-5
assert reldif( 66.2629056393085, r(n2min_x)) < 1e-5
assert reldif( 66.2629056393085, r(n1max_x)) < 1e-5

// 1B. "vertical tangents" = evr(9999), reg y on x, OLS
qui reg parentht childht
assert reldif( _b[childht], .3256475) < 1e-5
assert reldif( _b[_cons], 46.13535) < 1e-5
// OLS: parentht = 46.13535 + .3256475 * childht
ellip parentht childht, c(2.5) ylab(minmax) xlab(minmax) ///
	xline(68.08847) yline(68.30819) evr(9999) legend(off)	///
	plot(scatter parentht childht, mlabel(freq) m(i)) nograph
assert reldif( 64.10725496233079, r(amin_x)) < 1e-5
assert reldif( 72.06968469284163, r(amax_x)) < 1e-5
assert reldif( 67.01166541441368, r(amin_y)) < 1e-5
assert reldif( 69.60471389593114, r(amax_y)) < 1e-5
assert reldif( 68.08838804408605, r(bmin_x)) < 1e-5
assert reldif( 68.08855161108637, r(bmax_x)) < 1e-5
assert reldif( 65.7971288005925, r(bmin_y)) < 1e-5
assert reldif( 70.81925050975232, r(bmax_y)) < 1e-5
assert reldif( .3256604540673411, r(m1)) < 1e-5
assert reldif( -30703.75870056477, r(m2)) < 1e-5
assert reldif( .000032569302337, r(n)) < 1e-5
assert reldif( 9999, r(evr)) < 1e-12
assert reldif( 4.290308657623028, r(a)) < 1e-12
assert reldif( 2.330177065079445, r(b)) < 1e-12
assert reldif( 4.187009303811708, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( 2.5, r(c)) < 1e-5
assert reldif( .4599066315805654, r(theta)) < 1e-5
assert reldif( .7050144304027224, r(e2)) < 1e-5
assert reldif( .4587623682928201, r(r)) < 1e-5
assert reldif( 69.60343140425142, r(n2min_y)) < 1e-5	
assert reldif( 67.03152844313529, r(n1max_y)) < 1e-5
assert reldif( 65.78146052860485, r(n2min_x)) < 1e-5
assert reldif( 64.10737836779089, r(n1max_x)) < 1e-5

ellip parentht childht, c(2.5) ylab(minmax) xlab(minmax) ///
	xline(68.08847) yline(68.30819) evr(2) legend(off)	///
	plot(scatter parentht childht, mlabel(freq) m(i))
assert reldif( 68.30818965517241, r(mean_y)) < 1e-5
assert reldif( 68.08846982758621, r(mean_x)) < 1e-5 
assert reldif( 64.13493986737281, r(amin_x)) < 1e-5
assert reldif( 72.04199978779961, r(amax_x)) < 1e-5
assert reldif( 66.72511223054775, r(amin_y)) < 1e-5
assert reldif( 69.89126707979707, r(amax_y)) < 1e-5
assert reldif( 67.61977732315037, r(bmin_x)) < 1e-5
assert reldif( 68.55716233202205, r(bmax_x)) < 1e-5
assert reldif( 65.96719251935149, r(bmin_y)) < 1e-5
assert reldif( 70.64918679099334, r(bmax_y)) < 1e-5
assert reldif( 64.10737836779089, r(min_x)) < 1e-5	
assert reldif( 72.06968469368165, r(max_x)) < 1e-5
assert reldif( .400421254058035, r(m1)) < 1e-5
assert reldif( -4.994739863908748, r(m2)) < 1e-5
assert reldif( .2002106270290175, r(n)) < 1e-5
assert reldif( 2, r(evr)) < 1e-12
assert reldif( 4.290308657623028, r(a)) < 1e-12
assert reldif( 2.330177065079445, r(b)) < 1e-12
assert reldif( 4.258700890959737, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( 2.5, r(c)) < 1e-5
assert reldif( .4599066315805654, r(theta)) < 1e-5
assert reldif( .7050144304027224, r(e2)) < 1e-5
assert reldif( .4587623682928201, r(r)) < 1e-5
assert reldif( 68.40760677788619, r(n2min_y)) < 1e-5	
assert reldif( 68.23101276603715, r(n1max_y)) < 1e-5
assert reldif( 64.61736755836201, r(n2min_x)) < 1e-5
assert reldif( 71.57481601530354, r(n1max_x)) < 1e-5 

// 2. child height as function of parent height
// A. horizontal tangents
ellip childht parentht, c(2.5) ylab(minmax) xlab(minmax) ///
	xline(68.08847) yline(68.30819) evr(0) nograph
assert reldif( r(m1), 3.07080511368468) < 1e-5

// B. vertical tangents
qui reg childht parentht
assert reldif( _b[parentht], .6462906) < 1e-5
assert reldif( _b[_cons], 23.94153) < 1e-5
// OLS: childht = 23.94153 + .6462906 * parentht
ellip childht parentht, c(2.5) ylab(minmax) xlab(minmax) ///
	xline(68.08847) yline(68.30819) evr(9999) nograph
assert reldif( r(mean_y), 68.08846982758621) < 1e-5
assert reldif( r(mean_x), 68.30818965517241) < 1e-5
assert reldif( r(m1), .6463918736783243) < 1e-5


// Galton's Plate X data (sample data, N=314)
// Different data in Wachsmuth et al (2003), e.g. due to jittering?
// scatter jitterseed(#) option was added in Aug 2003
use galton2, clear
assert _N==58
qui expand freq
assert _N==314	// fweighted data
// 1. parent height as function of child height
// A. "horizontal tangents" = evr(0), regress x on y
ellip parentht childht, c(3.3) ylab(66(1)72) xlab(64(1)73)	///
	plot(scatter parentht childht, mlabel(freq) m(i))	///
	xline(68.25) yline(68.25) evr(0) legend(off)
	// m(i) -> marker symbol is invisible in legend
assert reldif( r(mean_y), 68.29458598726114) < 1e-5
assert reldif( r(mean_x), 68.17038216560509) < 1e-5
assert reldif( r(m1), 1.712762383207474) < 1e-5
assert reldif( r(e2), .6893720781316457) < 1e-5
	// e2 is fairly low which confirms Wachsmuth et al (2003)	
	

// B. "vertical tangents" = evr(9999), reg y on x, OLS
qui reg parentht childht
assert reldif( _b[childht], .260119) < 1e-5
assert reldif( _b[_cons], 50.56217) < 1e-5
// OLS: parentht = 50.56217 + .260119 * childht
ellip parentht childht, c(3.3) ylab(66(1)72) xlab(64(1)73)	///
	plot(scatter parentht childht, mlabel(freq) m(i))	///
	xline(68.25) yline(68.25) evr(9999) legend(off)
assert reldif( r(mean_y), 68.29458598726114) < 1e-5
assert reldif( r(mean_x), 68.17038216560509) < 1e-5
assert reldif( r(m1), .2601288529981) < 1e-5	// cf. above: 1.7
assert reldif( r(e2), .6893720781316457) < 1e-5

// C. "major axes" = evr(1)
ellip parentht childht, c(3.3) ylab(66(1)72) xlab(64(1)73)	///
	plot(scatter parentht childht, mlabel(freq) m(i))	///
	xline(68.25) yline(68.25)	
assert reldif( r(m1), .395677726907) < 1e-5
	

// 2. child height as function of parent height
// B. "vertical tangents" = evr(9999), reg y on x, OLS
qui reg childht parentht
assert reldif( _b[parentht], .5838522) < 1e-5
assert reldif( _b[_cons], 28.29644) < 1e-5
// OLS: childht = 28.29644 + .5838522 * parentht



* --------------------------- Example 8 (means) ----------------------
* Johnson and Wichern (2002, 181, 200, 221-223, 226-227) microwave oven
* radiation data -- Figure 5.2. (Figure 5.1 is too small.)
use oven, clear
qui mat accum Cov = x1 x2, noc means(m) dev
assert 42 == r(N)
mat input m2 = ( .564, .603 )
assert mreldif( m2, m) < 1e-3
mat S = Cov/(r(N)-1)
mat input S2 = ( /*
    */ .0144, .0117 \ /*
    */ .0117, .0146   /*
    */ )
assert mreldif( S2, S) < 1e-3

mat P = corr(S)
scalar r = P[1,2]
mat Sinv = syminv(S)
mat input S2inv = ( /*
    */  203.018, -163.391 \ /*
    */ -163.391,  200.228   /*
    */ )
assert mreldif( S2inv, Sinv) < 1e-2

matrix symeigen X lambda = S
mat input lambda2 = (.026,.002)
* Typo on p. 222: should be -.704 and not -.710
mat input X2 = (.704,.710 \ .710, -.704)
assert mreldif(lambda2, lambda) < 1e-3
assert mreldif(X2, X) < 1e-3

tempvar yvar xvar n
keep x2 x1

// display first half--arc of ellipse, run clockwise
ellip x2 x1, means c(t2) from(0) to(3.141592653589) npoints(10000)	

// display second half-arc of the ellipse, run counter-clockwise
ellip x2 x1, means c(t2) from(6.28318530) to(3.141592653589) npoints(10000)
 
// equation of ellipse is parametric with approximate points: 
// max_y, xmax_y, max_x, ymax_x, ymin_x, min_x, min_y, and xmin_y
// equation of intersecting line is algebraic with exact points: 
// amax_y, amax_x, amin_x, amin_y

// accuracy of parametric points changes with from(), to(), npoints():
ellip x2 x1, means c(t2) evr(0) npoints(20) nograph //
assert reldif( r(ymax_x), r(amax_x)) < 1e-3
rcof "noisily assert reldif( r(ymax_x), r(amax_x)) < 1e-4" == 9

ellip x2 x1, means c(t2) evr(0) npoints(400) nograph //
assert reldif( r(ymax_x), r(amax_x)) < 1e-4
rcof "noisily assert reldif( r(ymax_x), r(amax_x)) < 1e-5" == 9

ellip x2 x1, means c(t2) evr(0) npoints(1000000) nograph //
assert reldif( r(ymax_x), r(amax_x)) < 1e-7
rcof "noisily assert reldif( r(ymax_x), r(amax_x)) < 1e-8" == 9

ellip x2 x1, means c(t2) evr(0) ///
	from(5.65808191) to(5.65808208) npoints(1000000) nograph
assert reldif( r(ymax_x), r(amax_x)) < 1e-10
rcof "noisily assert reldif( r(ymax_x), r(amax_x)) < 1e-11" == 9


// test that evr() gives reasonable values
ellip x2 x1, means c(t2) evr(9999) ylab(minmax) xlab(minmax) ///
	yvarformat(%18.0g) xvarformat(%18.0g)
assert reldif( .6029812431051618, r(mean_y)) < 1e-5
assert reldif( .5642575139091128, r(mean_x)) < 1e-5	 
assert reldif( .516680286, r(amin_x)) < 1e-5
assert reldif( .611834741, r(amax_x)) < 1e-5
assert reldif( .564138065, r(amin_y)) < 1e-5
assert reldif( .641824420, r(amax_y)) < 1e-5
assert reldif( r(mean_x), r(bmin_x)) < 1e-5
assert reldif( r(mean_x), r(bmax_x)) < 1e-5
assert reldif( .574953233, r(bmin_y)) < 1e-5
assert reldif( .631009252, r(bmax_y)) < 1e-5
assert reldif( r(amin_x), r(min_x)) < 1e-5	// min_x	
assert reldif( r(amax_x), r(max_x)) < 1e-5	// max_x
assert reldif( .816423734, r(m1)) < 1e-5	// best evr-slope
assert reldif( -12247.3166609982, r(m2)) < 1e-5
assert reldif( .000081650538455, r(n)) < 1e-5
assert reldif( 9999, r(evr)) < 1e-12
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( .0207587712052785, r(b)) < 1e-12
assert reldif( .0614197443237838, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5

ellip x2 x1, means c(t2) evr(5) ylab(minmax) xlab(minmax)
assert reldif( .6029812431051618, r(mean_y)) < 1e-5
assert reldif( .5642575139091128, r(mean_x)) < 1e-5
assert reldif( .516870508, r(amin_x)) < 1e-5	// > evr(999)
assert reldif( .611644519, r(amax_x)) < 1e-5	// < evr(999)
assert reldif( .561790719, r(amin_y)) < 1e-5	// < evr(999)
assert reldif( .644171766, r(amax_y)) < 1e-5	// > evr(999)
assert reldif( .5600073018494842, r(bmin_x)) < 1e-5
assert reldif( .5685077259687414, r(bmax_x)) < 1e-5
assert reldif( .5785332870604768, r(bmin_y)) < 1e-5
assert reldif( .6274291991498467, r(bmax_y)) < 1e-5
assert reldif( .8692366862612623, r(m1)) < 1e-5	// > evr(999)
assert reldif( -5.752173233168365, r(m2)) < 1e-5
assert reldif( .1738473372522524, r(n)) < 1e-5
assert reldif( 5, r(evr)) < 1e-12
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( .0207587712052785, r(b)) < 1e-12
assert reldif( .0627868415040987, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5

ellip x2 x1, means c(t2) evr(1) ylab(minmax) xlab(minmax)	// orth reg
assert reldif( .6029812431051618, r(mean_y)) < 1e-5
assert reldif( .5642575139091128, r(mean_x)) < 1e-5
assert reldif( .519021073, r(amin_x)) < 1e-5	// > evr(5)
assert reldif( .609493954, r(amax_x)) < 1e-5	// < evr(5)
assert reldif( .557366638, r(amin_y)) < 1e-5	// < evr(5)
assert reldif( .648595847, r(amax_y)) < 1e-5	// > evr(5)
assert reldif( .549517874, r(bmin_x)) < 1e-5
assert reldif( .578997153, r(bmax_x)) < 1e-5
assert reldif( .588363801, r(bmin_y)) < 1e-5
assert reldif( .617598685, r(bmax_y)) < 1e-5
assert reldif( 1.008359715424517, r(m1)) < 1e-5		// > evr(5)
assert reldif( -.9917095900434719, r(m2)) < 1e-5
assert reldif( 1.008359715424517, r(n)) < 1e-5
assert reldif( 1, r(evr)) < 1e-12
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( .0207587712052785, r(b)) < 1e-12
assert reldif( .064241946899, r(a_evr)) < 1e-12
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5

ellip x2 x1, means c(t2) evr(.5) ylab(minmax) xlab(minmax)
assert reldif( .521063614, r(amin_x)) < 1e-5	// > evr(1)
assert reldif( .607451413, r(amax_x)) < 1e-5	// < evr(1) 
assert reldif( .55596631, r(amin_y)) < 1e-5	// < evr(1)
assert reldif( .64999616, r(amax_y)) < 1e-5	// > evr(1)
assert reldif( .544310594943189, r(bmin_x)) < 1e-5
assert reldif( .5842044328750367, r(bmax_x)) < 1e-5
assert reldif( .5938183526680035, r(bmin_y)) < 1e-5
assert reldif( .61214413354232, r(bmax_y)) < 1e-5
assert reldif( 1.08846215, r(m1)) < 1e-5	// > evr(1)
assert reldif( -.459363696, r(m2)) < 1e-5
assert reldif( 2.176924312554593, r(n)) < 1e-5
assert reldif( .5, r(evr)) < 1e-12
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( .0207587712052785, r(b)) < 1e-12
assert reldif( .0638445, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5	

ellip x2 x1, means c(t2) evr(.001) ylab(minmax) xlab(minmax)
assert reldif( .6029812431051618, r(mean_y)) < 1e-5
assert reldif( .5642575139091128, r(mean_x)) < 1e-5
assert reldif( .525663987, r(amin_x)) < 1e-5	// > evr(.5)
assert reldif( .602851040, r(amax_x)) < 1e-5	// < evr(.5)
assert reldif( .555081752, r(amin_y)) < 1e-9	// < evr(.5)
assert reldif( .650880734, r(amax_y)) < 1e-9	// > evr(.5)
assert reldif( .5364342810495593, r(bmin_x)) < 1e-5
assert reldif( .5920807467686664, r(bmax_x)) < 1e-5
assert reldif( .6029588254000785, r(bmin_y)) < 1e-5
assert reldif( .603003660810245, r(bmax_y)) < 1e-5
assert reldif( 1.24113, r(m1)) < 1e-5		// > evr(.5)
assert reldif( -.000805718, r(m2)) < 1e-5
assert reldif( 1241.127615703312, r(n)) < 1e-5
assert reldif( .001, r(evr)) < 1e-12
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( .0207587712052785, r(b)) < 1e-12
assert reldif( .06151278, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5

ellip x2 x1, means c(t2) evr(0) ylab(minmax) xlab(minmax) ///
	yvarformat(%18.0g) xvarformat(%18.0g)
assert reldif( .6029812431051618, r(mean_y)) < 1e-5
assert reldif( .5642575139091128, r(mean_x)) < 1e-5
assert reldif( r(amin_y), r(min_y)) < 1e-5	// min_y	
assert reldif( r(amax_y), r(max_y)) < 1e-5	// max_y
assert reldif( .525677013, r(amin_x)) < 1e-5	// > evr(.0001)
assert reldif( .602838014, r(amax_x)) < 1e-5	// < evr(.0001)
assert reldif( .555081746, r(amin_y)) < 1e-9	// < evr(.0001)
assert reldif( .650880739, r(amax_y)) < 1e-9	// > evr(.0001)
assert reldif( .536416221, r(bmin_x)) < 1e-5
assert reldif( .592098806, r(bmax_x)) < 1e-5
assert reldif( r(mean_y), r(bmin_y)) < 1e-12
assert reldif( r(mean_y), r(bmax_y)) < 1e-12
assert reldif( 1.24154679, r(m1)) < 1e-5		// > evr(.0001)
assert reldif( 0, r(m2)) < 1e-12
assert . == r(n)
assert reldif( 0, r(evr)) < 1e-12
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( .0207587712052785, r(b)) < 1e-12
assert reldif( .061504607, r(a_evr)) < 1e-7
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
// cannot divide by (0): n = -1 / (evr/m1), where evr = 0
rcof "noisily assert reldif( r(m1) / r(n), r(evr)) < 1e-5" == 9
assert reldif( -1 / r(m2), r(n)) < 1e-5		// n = .

// inscribing lines
use oven, clear
tempname mean_y mean_x r
scalar mean_y = r(mean_y)
scalar mean_x = r(mean_x)
scalar r = r(r)

// see [P] page 337-338
tempvar n1 n2 y1_ey y2_ey  n1_y n2_y n1_x n2_x a5_y a5_x b5_y b5_x
tempvar y x y_x x_y a_y a_x n b_y b_x b a bi_y bi_x
tempname n1_y n2_y n1max_y n2max_y n1min_y n2min_y  ///
	n1max_x n2min_x n2max_x n2max_y ///
	amin_y amin_x amax_x amax_y n

ellip x2 x1, means c(t2) evr(5) gen(ey ex) npoints(9999)	// evr(5)
assert reldif( .8692366862612623, r(m1)) < 1e-5	//
assert reldif( .1738473372522524, r(n)) < 1e-5	// 
assert r(n) < r(m1)

// n1 and n2 lines
assert reldif( .5685741619803201, r(n1max_y)) < 1e-5
assert reldif( .5559140335017324, r(n1max_x)) < 1e-5
assert reldif( .6373883231707376, r(n2min_y)) < 1e-5 
assert reldif( .5726009928276153, r(n2min_x)) < 1e-5

scalar `n' = r(n)
scalar `amin_y' = r(amin_y)
scalar `amin_x' = r(amin_x)
scalar `amax_y' = r(amax_y)
scalar `amax_x' = r(amax_x)
scalar `n1max_y' = r(n1max_y)
scalar `n1max_x' = r(n1max_x)
scalar `n2min_y' = r(n2min_y)
scalar `n2min_x' = r(n2min_x)

// inscribing a5 line (positive)
gen `a5_y' = `amin_y'			// min y (1)
replace `a5_y' = `amax_y' in 2/2	// max y (2)
gen `a5_x' = `amin_x'			// min x (1)
replace `a5_x' = `amax_x' in 2/2	// max x (2)

// inscribing b5 line (negative)
gen `b5_y' = r(bmin_y)			// min y (1)
replace `b5_y' = r(bmax_y) in 2/2	// max y (2)
gen `b5_x' = r(bmax_x)			// max x (1)
replace `b5_x' = r(bmin_x) in 2/2	// min x (2)

// inscribing n1 line (positive)
gen `n1_y' = `amin_y'			// min y (1)
replace `n1_y' =  `n1max_y' in 2/2 	// max y (2)
gen `n1_x' = `amin_x'			// min x (1)
replace `n1_x' =  `n1max_x' in 2/2	// max x (2)

// inscribing n2 line (positive)
gen `n2_y' = `n2min_y'			// min y (1)
replace `n2_y' =   `amax_y' in 2/2	// max y (2)
gen `n2_x' = `n2min_x'			// min x (1)
replace `n2_x' =  `amax_x' in 2/2	// max x (2)

ellip x2 x1, means c(t2) nograph			// evr(1)
assert r(n) == r(m1)

// inscribing xreg or evr(0) line(positive)
gen `y' = r(min_y)			// min y (1) ah_y
replace `y' = r(max_y) in 2/2		// max y (2)
gen `y_x' = r(ymin_x)			// min x (1) av_x
replace `y_x' = r(ymax_x) in 2/2	// max x (2)

// inscribing yreg or evr(infinite) line (positive)
gen `x' = r(min_x)			// min x (1) ah_x
replace `x' = r(max_x) in 2/2		// max x (2)
gen `x_y' = r(xmin_y)			// min y (1) av_y
replace `x_y' = r(xmax_y) in 2/2	// max y (2)

// inscribing a line (positive)
gen `a_y' = r(amin_y)			// min y (1)
replace `a_y' = r(amax_y) in 2/2	// max y (2)
gen `a_x' = r(amin_x)			// min x (1)
replace `a_x' = r(amax_x) in 2/2	// max x (2)

// inscribing b line (negative)
gen `b_y' = r(bmin_y)			// min y (1)
replace `b_y' = r(bmax_y) in 2/2	// max y (2)
gen `b_x' = r(bmax_x)			// max x (1)
replace `b_x' = r(bmin_x) in 2/2	// min x (2)

ellip x2 x1, means c(t2) legend(off) evr(0)		// evr(0)
tempvar b0_y b0_x
// inscribing b line (negative)
gen `b0_y' = r(bmin_y)			// min y (1)
replace `b0_y' = r(bmax_y) in 2/2	// max y (2)
gen `b0_x' = r(bmax_x)			// max x (1)
replace `b0_x' = r(bmin_x) in 2/2	// min x (2)


ellip x2 x1, means c(t2) legend(off) evr(.5)		// evr(.5) NEW

// n1 and n2 lines
assert reldif( .6174208578825305, r(n1max_y)) < 1e-5	// new
assert reldif( .5493417434199649, r(n1max_x)) < 1e-5	// new
assert reldif( .5889016720318649, r(n2min_y)) < 1e-5 
assert reldif( .5795285401362242, r(n2min_x)) < 1e-5

tempname n n3max_ya n3max_xa n4min_ya n4min_xa n3_y n4_y 
tempname n3max_y n3max_x n4min_y n4min_x
// calculate n3_y n3_x n4_y n4_x as normal "n1_" "n2" for evr(.5)
tempvar n3 n4 n3_y n3_x n4_y n4_x y3_ey y4_ey a05_y a05_x b05_y b05_x

scalar `n' = r(n)
scalar `amin_y' = r(amin_y)
scalar `amin_x' = r(amin_x)
scalar `amax_y' = r(amax_y)
scalar `amax_x' = r(amax_x)

scalar `n3max_ya' = r(n1max_y)
scalar `n3max_xa' = r(n1max_x)
scalar `n4min_ya' = r(n2min_y)
scalar `n4min_xa' = r(n2min_x)

// inscribing a05 line (positive)
gen `a05_y' = r(amin_y)			// min y (1)
replace `a05_y' = r(amax_y) in 2/2	// max y (2)
gen `a05_x' = r(amin_x)			// min x (1)
replace `a05_x' = r(amax_x) in 2/2	// max x (2)

// inscribing b05 line (negative)
gen `b05_y' = r(bmin_y)			// min y (1)
replace `b05_y' = r(bmax_y) in 2/2	// max y (2)
gen `b05_x' = r(bmax_x)			// max x (1)
replace `b05_x' = r(bmin_x) in 2/2	// min x (2)

// inscribing n1 line (positive)
gen `n3_y' = `amin_y'			// min y (1)
replace `n3_y' =  `n3max_ya' in 2/2 	// max y (2)
gen `n3_x' = `amin_x'			// min x (1)
replace `n3_x' =  `n3max_xa' in 2/2	// max x (2)

// inscribing n2 line (positive)
gen `n4_y' = `n4min_ya'			// min y (1)
replace `n4_y' =   `amax_y' in 2/2	// max y (2)
gen `n4_x' = `n4min_xa'			// min x (1)
replace `n4_x' =  `amax_x' in 2/2	// max x (2)


ellip x2 x1, means c(t2) legend(off) evr(9999)	// evr(9999)
// inscribing b line (negative)
gen `bi_y' = r(bmin_y)			// min y (1)
replace `bi_y' = r(bmax_y) in 2/2	// max y (2)
gen `bi_x' = r(bmax_x)			// max x (1)	
replace `bi_x' = r(bmin_x) in 2/2	// min x (2)

gen `n' = _n
ellip x2 x1, means c(t2) legend(off) 			/// evr(1)
	ylab(minmax, format(%9.4f)) xlab(minmax, format(%9.4f)) ///
	ylabel(.54(.04).66) xlabel(.51(.02).61)			///
	ytick(.56(.04).64) xtick(.52(.02).60)			///
	plot(line `y' `y_x', sort(`n') clpat(solid)	||	/// xreg, evr(0)
	line `x_y' `x', sort(`n') clpat(solid)	||		/// yreg, evr(i)
	line `a_y' `a_x', sort(`n') clpat(solid)	||	/// evr(1)
	line `a5_y' `a5_x', sort(`n') clpat(dot) ||	/// major for evr(5)
	line `a05_y' `a05_x', sort(`n') clpat(dot) ||	/// major for evr(0.5)
	line `b0_y' `b0_x', sort(`n') clpat(solid) ||	/// minor for evr(0)
	line `bi_y' `bi_x', sort(`n') clpat(solid) ||	/// minor for evr(9999)
	line `b_y' `b_x', sort(`n') clpat(solid) ||	/// minor for evr(1)
	line `b05_y' `b05_x', sort(`n') clpat(dot) ||	/// minor for evr(0.5)
	line `b5_y' `b5_x', sort(`n') clpat(dot) ||	/// minor for evr(5)
	line `n1_y' `n1_x', sort(`n') clpat(solid) ||	/// n1 for evr(5)
	line `n2_y' `n2_x', sort(`n') clpat(solid) ||	/// n2 for evr(5)
	line `n3_y' `n3_x', sort(`n') clpat(solid) ||	/// n1 for evr(.5)
	line `n4_y' `n4_x', sort(`n') clpat(solid))	/// n2 for evr(.5)
	saving(fig8.gph, replace)                 /* 8 */	
assert reldif( .564, r(mean_x)) < 1e-3
assert reldif( .516, r(min_x)) < 1e-3
assert reldif( .612, r(max_x)) < 1e-3
assert reldif( .603, r(mean_y)) < 1e-3
assert reldif( .555, r(min_y)) < 1e-3
assert reldif( .651, r(max_y)) < 1e-3
assert reldif( .064241946899377, r(a)) < 1e-12
assert reldif( 6.62/42, r(c)) < 1e-3
assert reldif( r(n), -1 / r(m2)) < 1e-5
assert reldif( r(n), r(m1)) < 1e-5	// for evr(1)


// simplified figure for the article (?): geometry of linear regression
// scatteri() to label m and n in graph?
ellip x2 x1, means c(t2) legend(off) g(y x) ///
	ylab(minmax, format(%9.4f)) xlab(minmax, format(%9.4f)) ///
	plot(line `a05_y' `a05_x', sort(`n') clpat(solid) ||	/// major for evr(0.5)
	line `n4_y' `n4_x', sort(`n') clpat(solid) ||	/// n2 for evr(.5)
	line `a_y' `a_x', sort(`n') clpat(dash))		// evr(1)

// graph graphics commands allow more flexibility	
// For example, -graph twoway area- allows shading

// graph twoway (line y x)	///
// (area y x, nodropbase bfc(gray))

// but the shading covers everything else
// how can you shade subsets of the ellipse such as the "a" and "b" parts?
// an example was mentioned on Statalist mid-October 2003.



* --------------------------- Example 9 (means) ----------------------
* Johnson and Wichern (2002, 239-242, 248-249) appearances overtime
* data -- Figure 5.6 (p. 261) and Figure 5.9 (p. 265)
use overtime, clear

qui mat accum Cov = x y, noc means(m) dev
assert 16 == r(N)
mat input m2 = ( 3558, 1478 )
assert mreldif( m2, m) < 1e-3
mat S = Cov/(r(N)-1)
mat input S2 = ( /*
    */ 367884.7,  -72093.8 \ /*
    */ -72093.8, 1399053.1   /*
    */ )
assert mreldif( S2, S) < 1e-5

* 99% chi2 ellipse with datapoints
// how to add an arrow pointing at the outlier?
assert y==5326 & x==3135 if y>5000 & !mi(y)
ellip y x, l(99) c(chi2) legend(off) ///
	plot(scatter y x, msymbol(o) ///
		text(5326 3135 "Madison protest", place(e)) ///
		|| scatteri `=r(mean_y)' `=r(mean_x)', msymbol(+))  ///
	ylabel(-2000(1000)5000) xlabel(1500(1000)5500) ///
	ysize(2.6) xsize(1.89)				/* 9a */
assert reldif( 3558, r(mean_x)) < 1e-3
assert reldif( 1478, r(mean_y)) < 1e-3
assert reldif( 9.21, r(c)) < 1e-3
local Chi = r(c)

* codify the values I think are right
assert reldif( 1717.06, r(min_x)) < 1e-4
assert reldif( 5398.46, r(max_x)) < 1e-4
assert reldif( -2111.18, r(min_y)) < 1e-4
assert reldif( 5068.10, r(max_y)) < 1e-4

// diameter() dlopts() options
ellip y x, l(99) c(chi2) evr(0) diameter(5) dlopts(conn(l) clpat(dot)) legend(off)
ellip y x, l(99) c(chi2) evr(0) diameter(-5) legend(off)
ellip y x, l(99) c(chi2) evr(0) diameter(123456) legend(off)
ellip y x, l(99) c(chi2) evr(0) diameter(1234567) legend(off)
// diameter(123456) is ignored
assert reldif( 3372.772560062504, r(amin_x)) < 1e-5
assert reldif( 3742.727439937496, r(amax_x)) < 1e-5 
assert reldif( -2111.235870110533, r(amin_y)) < 1e-5
assert reldif( 5068.10186764545, r(amax_y)) < 1e-5
assert reldif( 1726.322426120029, r(bmin_x)) < 1e-5
assert reldif( 5389.177573879971, r(bmax_x)) < 1e-5
assert reldif( r(mean_y), r(bmin_y)) < 1e-12
assert reldif( r(mean_y), r(bmax_y)) < 1e-12
assert reldif( -19.40600632879056, r(m1)) < 1e-5
assert reldif( 0, r(m2)) < 1e-5
assert reldif( ., r(n)) < 1e-5
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
// cannot divide by (0): n = -1 / (evr/m1), where evr = 0
rcof "noisily assert reldif( r(m1) / r(n), r(evr)) < 1e-5" == 9
assert reldif( -1 / r(m2), r(n)) < 1e-5

tempvar n1 n2 y1_ey y2_ey  n1_y n2_y n1_x n2_x a5_y a5_x b5_y b5_x
tempvar y x y_x x_y a_y a_x n b_y b_x b a bi_y bi_x a05_y a05_x b05_y b05_x
tempname n1_y n2_y n1max_y n2max_y n1min_y n2min_y  ///
	n1max_x n2min_x n2max_x n2max_y n1min_x ///
	amin_y amin_x amax_x amax_y n


// ------------------------------------------------------ evr(5)
// few iterations to get less reliable 2nd intersections for normals
ellip y x, l(99) c(chi2) evr(5) diameter(-.7310623840245107) np(20)
assert reldif( 2296.919613689247, r(n2min_y)) < 1e-5
assert reldif( 1246.395744699814, r(n1max_y)) < 1e-5 
assert reldif( 5298.758732798908, r(n2max_x)) < 1e-5
assert reldif( 1742.109972301507, r(n1min_x)) < 1e-5

// default iterations to get reliable 2nd intersections for normals
ellip y x, l(99) c(chi2) evr(5) diameter(-.7310623840245107)
assert reldif( 1783.250583538912, r(amin_x)) < 1e-5
assert reldif( 5332.249416461088, r(amax_x)) < 1e-5 
assert reldif( 181.1677261518537, r(amin_y)) < 1e-5
assert reldif( 2775.707273848147, r(amax_y)) < 1e-5
assert reldif( 3068.367294305408, r(bmin_x)) < 1e-5
assert reldif( 4047.132705694592, r(bmax_x)) < 1e-5
assert reldif( -1868.627787373506, r(bmin_y)) < 1e-5
assert reldif( 4825.502787373506, r(bmax_y)) < 1e-5
assert reldif( -.7310623840245107, r(m1)) < 1e-5	// negative
assert reldif( 6.839361604785238, r(m2)) < 1e-5		// positive
assert reldif( 5, r(evr)) < 1e-12
assert reldif( -.1462124768049022, r(n)) < 1e-5		// negative
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5
assert reldif( 2241.781643213138, r(n2min_y)) < 1e-5
assert reldif( 687.4998547388127, r(n1max_y)) < 1e-5 
assert reldif( 5307.954489651162, r(n2max_x)) < 1e-5
assert reldif( 1812.089278273983, r(n1min_x)) < 1e-5

// many iterations to get more reliable 2nd intersections for normals
ellip y x, l(99) c(chi2) evr(5) diameter(-.7310623840245107) npoints(4000)
assert reldif( 2261.327718576734, r(n2min_y)) < 1e-5
assert reldif( 698.2996655346608, r(n1max_y)) < 1e-5 
assert reldif( 5304.74789277508, r(n2max_x)) < 1e-5
assert reldif( 1810.297040812181, r(n1min_x)) < 1e-5
assert reldif( -.7310623840245107, r(m1)) < 1e-5

// very many iterations to get most reliable 2nd intersections for normals
ellip y x, l(99) c(chi2) evr(5) diameter(-.7310623840245107) npoints(1000000)
assert reldif( 2260.805985662553, r(n2min_y)) < 1e-5
assert reldif( 696.0580081450399, r(n1max_y)) < 1e-5 
assert reldif( 5304.83424263159, r(n2max_x)) < 1e-5
assert reldif( 1810.667578528059, r(n1min_x)) < 1e-5
assert reldif( -.7310623840245107, r(m1)) < 1e-5

// tempname n amin_y amin_x amax_y amax_x
scalar `n' = r(n)
scalar `amin_y' = r(amin_y)
scalar `amin_x' = r(amin_x)
scalar `amax_y' = r(amax_y)
scalar `amax_x' = r(amax_x)

tempname n1max_ya n1min_xa n2min_ya n2max_xa
scalar `n1max_ya' = r(n1max_y)
scalar `n1min_xa' = r(n1min_x)
scalar `n2min_ya' = r(n2min_y)
scalar `n2max_xa' = r(n2max_x)

// inscribing a5 line (negative) 
gen `a5_y' = `amin_y'			// min y (1)
qui replace `a5_y' = `amax_y' in 2/2	// max y (2)
gen `a5_x' = `amax_x'			// max x (1)
qui replace `a5_x' = `amin_x' in 2/2	// min x (2)

// inscribing n1 line (negative)
gen `n1_y' = `amin_y'			// min y (1)
qui replace `n1_y' =  `n1max_ya' in 2/2 // max y (2)
gen `n1_x' =  `amax_x'			// max x (1)
qui replace `n1_x' = `n1min_xa' in 2/2	// min x (2)


gen `n' = _n

// inscribing b_5 line (positive)
gen `b5_y' = r(bmin_y)			// min y (1)
qui replace `b5_y' = r(bmax_y) in 2/2	// max y (2)
gen `b5_x' = r(bmin_x)			// min x (1)
qui replace `b5_x' = r(bmax_x) in 2/2	// max x (2)


// ------------------------------------------------------ evr(0)
ellip y x, means l(99) c(chi2) legend(off) evr(0)
assert reldif( 1726.322426120029, r(bmin_x)) < 1e-5
assert reldif( 5389.177573879971, r(bmax_x)) < 1e-5
assert reldif( 1478.4375, r(bmin_y)) < 1e-5
assert reldif( 1478.4375, r(bmax_y)) < 1e-5

// inscribing b line (horizontal line -- zero slope)
tempvar b0_y b0_x
gen `b0_y' = r(bmin_y)			// min y (1)
qui replace `b0_y' = r(bmax_y) in 2/2	// max y (2)
gen `b0_x' = r(bmax_x)			// max x (1)
qui replace `b0_x' = r(bmin_x) in 2/2	// min x (2)


// ------------------------------------------------------ evr(9999)
ellip y x, means l(99) c(chi2) legend(off) evr(9999)
assert reldif( 1717.004605669095, r(amin_x)) < 1e-5
assert reldif( 5398.495394330906, r(amax_x)) < 1e-5 
assert reldif( 1117.573537145922, r(amin_y)) < 1e-5
assert reldif( 1839.301462854078, r(amax_y)) < 1e-5
assert reldif( 3557.679976709794, r(bmin_x)) < 1e-5
assert reldif( 3557.820023290206, r(bmax_x)) < 1e-5
assert reldif( -2093.051279821575, r(bmin_y)) < 1e-5
assert reldif( 5049.926279821575, r(bmax_y)) < 1e-5
assert reldif( -.1960423011055522, r(m1)) < 1e-5
assert reldif( 51004.29827446469, r(m2)) < 1e-5
assert reldif( -.0000196061907296, r(n)) < 1e-5
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5

// inscribing b line (vertical line --- evr(9999) slope)
tempvar bi_y bi_x
gen `bi_y' = r(bmin_y)			// min y (1)
qui replace `bi_y' = r(bmax_y) in 2/2	// max y (2)
gen `bi_x' = r(bmax_x)			// max x (1)
qui replace `bi_x' = r(bmin_x) in 2/2	// min x (2)


// ------------------------------------------------------ evr(.5)
ellip y x, l(99) c(chi2) evr(.5) diameter(-16.88418927743826)
assert reldif( 3345.168577186066, r(amin_x)) < 1e-5
assert reldif( 3770.331422813934, r(amax_x)) < 1e-5 
assert reldif( -2110.827479657619, r(amin_y)) < 1e-5
assert reldif( 5067.70247, r(amax_y)) < 1e-5
assert reldif( 1729.320961090339, r(bmin_x)) < 1e-5
assert reldif( 5386.17903890966, r(bmax_x)) < 1e-5
assert reldif( 1424.291310068544, r(bmin_y)) < 1e-5
assert reldif( 1532.583689931456, r(bmax_y)) < 1e-5
assert reldif( -16.88418927743826, r(m1)) < 1e-5	// negative
assert reldif( .0296135036029316, r(m2)) < 1e-5		// positive
assert reldif( -33.76837855487673, r(n)) < 1e-5
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5
assert reldif( 5051.332450965303, r(n1max_y)) < 1e-5
assert reldif( 3550.503, r(n1min_x)) < 1e-5	// new
assert reldif( -2091.617205819832, r(n2min_y)) < 1e-5 
assert reldif( 3550.503311686258, r(n2max_x)) < 1e-5

// tempname n
scalar `n' = r(n)
scalar `amin_y' = r(amin_y)
scalar `amin_x' = r(amin_x)
scalar `amax_y' = r(amax_y)
scalar `amax_x' = r(amax_x)

tempname n1max_ya n1min_xa n2min_ya n2max_xa
scalar `n1max_ya' = r(n1max_y)
scalar `n1min_xa' = r(n1min_x)
scalar `n2min_ya' = r(n2min_y)
scalar `n2max_xa' = r(n2max_x)

tempvar a05_y a05_x b05_y b05_x n1_y n1_x n2_y n2_x

// inscribing a05 line (negative) 
gen `a05_y' = `amin_y'			// min y (1)
qui replace `a05_y' = `amax_y' in 2/2	// max y (2)
gen `a05_x' = `amax_x'			// max x (1)
qui replace `a05_x' = `amin_x' in 2/2	// min x (2)

// inscribing b05 line (positive)
gen `b05_y' = r(bmin_y)			// min y (1)
qui replace `b05_y' = r(bmax_y) in 2/2	// max y (2)
gen `b05_x' = r(bmin_x) 		// min x (1)
qui replace `b05_x' = r(bmax_x)	in 2/2	// max x (2)

// inscribing n1 line (negative)
gen `n1_y' = `amin_y'			// min y (1)
qui replace `n1_y' =  `n1max_ya' in 2/2 // max y (2)
gen `n1_x' =  `amax_x'			// max x (1)
qui replace `n1_x' = `n1min_xa' in 2/2	// min x (2)

// inscribing n2 line (negative)
gen `n2_y' = `n2min_ya'			// min y (1)
qui replace `n2_y' = `amax_y' in 2/2	// max y (2)
gen `n2_x' =  `n2max_xa'		// max x (1)
qui replace `n2_x' = `amin_x' in 2/2	// min x (2)


ellip y x, means c(chi2) l(99) nograph	// evr(1)
assert reldif( 3308.150088806973, r(amin_x)) < 1e-5
assert reldif( 3807.349911193027, r(amax_x)) < 1e-5 
assert reldif( -2108.99250116141, r(amin_y)) < 1e-5
assert reldif( 5065.86750116141, r(amax_y)) < 1e-5
assert reldif( 1734.005638532051, r(bmin_x)) < 1e-5
assert reldif( 5381.494361467949, r(bmax_x)) < 1e-5
assert reldif( 1351.548214212176, r(bmin_y)) < 1e-5
assert reldif( 1605.326785787824, r(bmax_y)) < 1e-5
assert reldif( -14.37272146458049, r(m1)) < 1e-5
assert reldif( .0695762457001867, r(m2)) < 1e-5
assert reldif( -14.37272146458049, r(n)) < 1e-5
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5
assert r(n) == r(m1)

// inscribing b line (positive)
gen `b_y' = r(bmin_y)			// min y (1)
qui replace `b_y' = r(bmax_y) in 2/2	// max y (2)
gen `b_x' = r(bmin_x) 			// min x (1)
qui replace `b_x' = r(bmax_x)	in 2/2	// max x (2)

// inscribing xreg or evr(0) line(positive)
gen `y' = r(min_y)			// min y (1) ah_y
qui replace `y' = r(max_y) in 2/2	// max y (2)
gen `y_x' = r(ymin_x)			// min x (1) av_x
qui replace `y_x' = r(ymax_x) in 2/2	// max x (2)

// inscribing yreg or evr(infinite) line (positive)
gen `x' = r(min_x)			// min y (1) ah_x
qui replace `x' = r(max_x) in 2/2	// max y (2)
gen `x_y' = r(xmin_y)			// min x (1) av_y
qui replace `x_y' = r(xmax_y) in 2/2	// max x (2)

ellip y x, means c(chi2) l(99) legend(off) ///				evr(1)
	ylab(minmax, format(%9.0f)) xlab(minmax, format(%9.0f)) ///
	ylabel(-2000(1000)5000) xlabel(1500(1000)5500)		///
	ytick(-2500(1000)4500) xtick(2000(1000)5000)		///
	plot(line `y' `y_x', sort(`n') clpat(solid)	||	/// xreg, evr(0)
	line `x_y' `x', sort(`n') clpat(solid)		||	/// yreg, evr(i)
	line `a5_y' `a5_x', sort(`n') clpat(solid)	||	/// a evr(5)
	line `b5_y' `b5_x', sort(`n') clpat(solid)	||	/// b evr(5)
	line `a05_y' `a05_x', sort(`n') clpat(solid)	||	/// a evr(.5)
	line `b_y' `b_x', sort(`n') clpat(solid)	||	/// b evr(1)
	line `b05_y' `b05_x', sort(`n') clpat(solid)	||	/// b evr(.5)
	line `bi_y' `bi_x', sort(`n') clpat(solid)	||	/// b evr(9999)
	line `b0_y' `b0_x', sort(`n') clpat(solid)	||	/// b evr(0)
	line `n1_y' `n1_x', sort(`n') clpat(dot)	||	/// n1
	line `n2_y' `n2_x', sort(`n') clpat(solid))		// n2		
assert reldif( 3308.150088806973, r(amin_x)) < 1e-5
assert reldif( 3807.34, r(amax_x)) < 1e-5 
assert reldif( -2108.99250116141, r(amin_y)) < 1e-5
assert reldif( 5065.86750116141, r(amax_y)) < 1e-5
assert reldif( 1734.005638532051, r(bmin_x)) < 1e-5
assert reldif( 5381.494361467949, r(bmax_x)) < 1e-5
assert reldif( 1351.548214212176, r(bmin_y)) < 1e-5
assert reldif( 1605.3266785787824, r(bmax_y)) < 1e-5
assert reldif( -14.37272146458049, r(m1)) < 1e-5	// negative
assert reldif( .0695762457001867, r(m2)) < 1e-5		// positive
assert reldif( 1, r(evr)) < 1e-12
assert reldif( -14.37272146458049, r(n)) < 1e-5		// negative
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5
assert reldif( 5066.574851889303, r(n1max_y)) < 1e-5 
assert reldif( 3319.280806348416, r(n1min_x)) < 1e-5


use overtime, clear
ellip y x, l(99) c(chi2) evr(4)
assert reldif( 2051.707177935609, r(amin_x)) < 1e-5
assert reldif( 5063.792822064391, r(amax_x)) < 1e-5 
assert reldif( -870.2286187177569, r(amin_y)) < 1e-5
assert reldif( 3827.103618717757, r(amax_y)) < 1e-5
assert reldif( 2499.365084309102, r(bmin_x)) < 1e-5
assert reldif( 4616.134915690898, r(bmax_x)) < 1e-5
assert reldif( -1236.248921461599, r(bmin_y)) < 1e-5
assert reldif( 4193.123921461599, r(bmax_y)) < 1e-5
assert reldif( -1.559494912301597, r(m1)) < 1e-5
assert reldif( 2.564933023152066, r(m2)) < 1e-5
assert reldif( -.3898737280753992, r(n)) < 1e-5
assert reldif( r(m1) * r(m2), -r(evr)) < 1e-5
assert reldif( r(m1) / r(n), r(evr)) < 1e-5
assert reldif( -1 / r(m2), r(n)) < 1e-5
assert reldif( 9.210340371976212, r(c)) < 1e-5
local Chi = r(c)


* verify that chi2_n gives a smaller ellipse than chi2
* see Paradowski (1997) for theoretical example
ellip y x, l(99) c(chi2_n)
assert reldif( 3557.75, r(mean_x)) < 1e-4
assert reldif( 1478.44, r(mean_y)) < 1e-4
assert reldif( 3097.57, r(min_x)) < 1e-4
assert reldif( 4017.94, r(max_x)) < 1e-4
assert reldif( 581.03, r(min_y)) < 1e-4
assert reldif( 2375.85, r(max_y)) < 1e-4
assert reldif( .5756462732485133, r(c)) < 1e-4
assert reldif( `Chi'/16, r(c)) < 1e-4

* verify that pchi2_n = ch2 * (n+1)/n, as in Chew (1966, 608)
ellip y x, l(99) c(pchi2_n)
assert reldif( 9.785986645224725, r(c)) < 1e-4
assert reldif( `Chi'*(16+1)/(16), r(c)) < 1e-4


use overtime, clear
qui keep if y ~= 5326
// 95% prediction ellipse without the outlier
ellip y x, c(pt2) legend(off)	/// 
	plot(scatter y x, msymbol(o)	///
	|| scatteri `=r(mean_y)' `=r(mean_x)', msymbol(+))  ///
	ylabel(-500(500)3000) xlabel(1500(1000)5500) ///	
	ysize(2.6) xsize(1.89)				/* 9b */
assert reldif( 1221.93, r(mean_y)) < 1e-3
assert reldif( 3585.93, r(mean_x)) < 1e-3
assert reldif( 1761.95, r(min_x)) < 1e-3
assert reldif( 5409.97, r(max_x)) < 1e-3
assert reldif( -579.26, r(min_y)) < 1e-3
assert reldif( 3023.14, r(max_y)) < 1e-3	



* --------------------------- Example 10 (means) ----------------------
* Batschelet (1981, 133, 142-143, 242, 257, 264-265) Example 7.2.1 data
* -- Figure 7.4.1
use batsch.dta, clear
qui summarize x
assert reldif( 5.1, r(mean)) < 1e-2
assert reldif( 2.09, r(sd)) < 1e-2
qui summarize y
assert reldif( 3.1, r(mean)) < 1e-2
assert reldif( 1.15, r(sd)) < 1e-2

* c(sd 1)
* Another testable example using c(sd 1) is:
* Dempster (1969, 127, 139, 150-151), Example 8.1.
ellip y x, means c(sd 1) g(var1 var2)

// Batschelet rounded his values to 2 or 3 digits
assert reldif( 5.099999952316284, r(mean_x)) < 1e-5
assert reldif( 3.006158389387289, r(min_x)) < 1e-5
assert reldif( 7.193906419658088, r(max_x)) < 1e-5
assert reldif( 2.093906467341804, r(s_x)) < 1e-5
assert reldif( 3.100000011920929, r(mean_y)) < 1e-5
assert reldif( 1.948222753634889, r(min_y)) < 1e-5
assert reldif( 4.251810102930659, r(max_y)) < 1e-5
assert reldif( 1.151810148615272, r(s_y)) < 1e-5
assert reldif( .6717022705866809, r(r)) < 1e-5
assert reldif( 1, r(c)) < 1e-12
assert reldif( .4071622291068654, r(theta)) < 1e-5
assert reldif( 2.2545704498407, r(a)) < 1e-5	// p. 137
assert reldif( .7924790212580208, r(b)) < 1e-5
* no certified value for lambda#, m#

* add c(sd 1)to 90% hotelling
ellip y x, means c(t2) level(90)  /*
 */ plot(line var1 var2) conn(ll) sy(..) ytick(1(1)5) xtick(1(1)8) /*
 */ ylab(1(1)5) xlab(1(1)8) nograph
assert reldif( 5.099999952316284, r(mean_x)) < 1e-5
assert reldif( 3.347601578804157, r(min_x)) < 1e-5
assert reldif( 6.852452646861169, r(max_x)) < 1e-5
assert reldif( 2.093906467341804, r(s_x)) < 1e-5
assert reldif( 3.100000011920929, r(mean_y)) < 1e-5
assert reldif( 2.136043316183701, r(min_y)) < 1e-5
assert reldif( 4.063984186453878, r(max_y)) < 1e-5
assert reldif( 1.151810148615272, r(s_y)) < 1e-5
assert reldif( .6717022705866809, r(r)) < 1e-5
assert reldif( .7004514690350296, r(c)) < 1e-5
assert reldif( .4071622291068654, r(theta)) < 1e-5
assert reldif( 1.886917167260295, r(a)) < 1e-5
assert reldif( .6632492987793088, r(b)) < 1e-5
* no certified value for lambda#, m#

* certify that a() + g() does not damage data
qui save batsch1.dta, replace
ellip y x, means c(t2) level(90) gen(var3 var4)  /*
 */ plot(line var1 var2) sy(..) ytick(1(1)5) xtick(1(1)8) /*
 */ ylab(1(1)5) xlab(1(1)8) saving(fig10.gph, replace)             /* 10 */
drop var3 var4
cf _all using batsch1.dta



* --------------------------- Example 11 (means) ----------------------
* Friendly (1991, 188) modified auto1.dta -- Figure 4.22
* Friendly used F(2,n-1) but I use F(2,n-2)
use auto, clear
sort foreign
qui replace foreign=2 if _n==56 | _n==57 | _n==58 | _n==59 | /*
   */ _n==61 | _n==62 | _n==63 | _n==66|  _n==67 | _n==68 | _n==69
label define origin 0 "American" 1 "European" 2 "Japanese", modify
qui save auto1.dta, replace

* generate the graphs that are to be overlaid
* codify the values I think are right
ellip weight price if foreign==0, level(50) c(f) gen(y0 x0) /*
    */ nograph /* A */
assert reldif( 2400.55, r(min_x)) < 1e-4
assert reldif( 9744.41, r(max_x)) < 1e-4
assert reldif( 6072.42, r(mean_x)) < 1e-4
assert reldif( 2492.7, r(min_y)) < 1e-4
assert reldif( 4141.55, r(max_y)) < 1e-4
assert reldif( 3317.12, r(mean_y)) < 1e-4

ellip weight price if foreign==1, level(50) c(f) gen(y1 x1) /*
    */ nograph  /* E */
assert reldif( 3717.92, r(min_x)) < 1e-4
assert reldif( 11369.5, r(max_x)) < 1e-4
assert reldif( 7543.64, r(mean_x)) < 1e-4
assert reldif( 1717.52, r(min_y)) < 1e-4
assert reldif( 3049.77, r(max_y)) < 1e-4
assert reldif( 2383.64, r(mean_y)) < 1e-4

ellip weight price if foreign==2, level(50) c(f) gen(y2 x2) /*
    */ nograph  /* J */
assert reldif( 3622.41, r(min_x)) < 1e-4
assert reldif( 6829.09, r(max_x)) < 1e-4
assert reldif( 5225.73, r(mean_x)) < 1e-4
assert reldif( 1885.89, r(min_y)) < 1e-4
assert reldif( 2610.47, r(max_y)) < 1e-4
assert reldif( 2248.18, r(mean_y)) < 1e-4
   
ellip weight price if foreign==2, c(f) l(50) legend(off)	///
	plot(scatter weight price if foreign==2, msymbol(x)	///
		text(2248.18 5225.73 "Japan", place(e)) ///
		|| scatteri 2248.18 5225.73, msymbol(+)	///
	|| line y0 x0	///
		|| scatter weight price if foreign==0, msymbol(oh)	///
		text(3317.12 6072.42 "America", place(e)) ///
		|| scatteri 3317.12 6072.42, msymbol(+)	///	
	|| line y1 x1  ///
		|| scatter weight price if foreign==1, msymbol(s)	///
		text(2383.64 7543.64 "Europe", place(e)) ///
		|| scatteri 2383.64 7543.64, msymbol(+))	///		
	ylabel(1000(1000)5000) xlabel(3000(2000)17000)	///
	saving(fig11.gph, replace)				/* 11 */


* --------------------------- Example 12 (means) ----------------------
* Iris data. Friendly (1991, 182) -- Figure 4.20
* Friendly used F(2,n-1) but I use F(2,n-2). See also SAS examples.
use iris, clear
ellip sepalwid petalwid, l(75) gen(y0 x0) c(f 2) /*
    */ nograph
assert reldif( 30.573, r(mean_y)) < 1e-4
assert reldif( 11.993, r(mean_x)) < 1e-4
assert reldif( -.758036, r(min_x)) < 1e-4
assert reldif( 24.7451, r(max_x)) < 1e-4
assert reldif( 23.2816, r(min_y)) < 1e-4
assert reldif( 37.8649, r(max_y)) < 1e-4

ellip sepalwid petalwid if species == "Setosa", /*
    */ l(75) gen(y1 x1) c(f 2) nograph
assert reldif( 34.28, r(mean_y)) < 1e-4
assert reldif( 2.46, r(mean_x)) < 1e-4   
assert reldif( .679622, r(min_x)) < 1e-4
assert reldif( 4.24043, r(max_x)) < 1e-4
assert reldif( 27.876, r(min_y)) < 1e-4
assert reldif( 40.6841, r(max_y)) < 1e-4

ellip sepalwid petalwid if species == "Versicolor", /*
    */ l(75) gen(y2 x2) c(f 2) nograph
assert reldif( 27.7, r(mean_y)) < 1e-4
assert reldif( 13.26, r(mean_x)) < 1e-4   
assert reldif( 9.91918, r(min_x)) < 1e-4
assert reldif( 16.6009, r(max_x)) < 1e-4
assert reldif( 22.3986, r(min_y)) < 1e-4
assert reldif( 33.0014, r(max_y)) < 1e-4

ellip sepalwid petalwid if species == "Virginica", /*
    */ l(75) gen(y3 x3) c(f 2) nograph
assert reldif( 29.74, r(mean_y)) < 1e-4
assert reldif( 20.26, r(mean_x)) < 1e-4
assert reldif( 15.6201, r(min_x)) < 1e-4
assert reldif( 24.9001, r(max_x)) < 1e-4
assert reldif( 24.2916, r(min_y)) < 1e-4
assert reldif( 35.1883, r(max_y)) < 1e-4

* cf. SJ02-04, p. 398-399: y = petal length, x = petal width 

ellip sepalwid petalwid, c(f) l(75) legend(off)	plot(line y1 x1	///
	|| scatter sepalwid petalwid if species == "Setosa", ///
		msymbol(oh) text(34.28 2.46 "Setosa", place(e)) ///
	|| scatteri 34.28 2.46, msymbol(+)	///	
	|| line y2 x2 ///
	|| scatter sepalwid petalwid if species == "Versicolor", ///
		msymbol(s) text(27.7 13.26 "Versicolor", place(e)) ///
	|| scatteri 27.7 13.26, msymbol(+)	///
	|| line y3 x3 ///
	|| scatter sepalwid petalwid if species == "Virginica", ///
		msymbol(x) text(29.74 20.26 "Virginica", place(e)) ///
	|| scatteri 29.74 20.26, msymbol(+))	///			
	ylabel(20(10)50) xlabel(0(10)30) ///
	saving(fig12a.gph, replace) 				/* 12a */ 

// SAS OnlineDoc: Version 8, iris data, -- Figure 40.35
use iris, clear
qui mat accum R = sepallen sepalwid petallen petalwid, nocons dev
mat R = corr(R)
mat input R2 = ( /*
    */  1.0000, -0.1176,  0.8718,  0.8179 \ /*
    */ -0.1176,  1.0000, -0.4284, -0.3661 \ /*
    */  0.8718, -0.4284,  1.0000,  0.9629 \ /*
    */  0.8179, -0.3661   0.9629,  1.0000   /*
    */ )
assert mreldif( R2, R) < 1e-3

ellip sepalwid sepallen, c(pt2) l(80) legend(off)	///
	plot(scatter sepalwid sepallen, msymbol(oh))	///				
	ylabel(20(10)40) xlabel(40(10)80)		///
	saving(fig12b.gph, replace)                                /* 12b */
assert reldif( 22.6579, r(min_y)) < 1e-4
assert reldif( 38.4887, r(max_y)) < 1e-4
assert reldif( 43.3959, r(min_x)) < 1e-4
assert reldif( 73.4713, r(max_x)) < 1e-4

ellip petallen sepallen, c(pt2) l(80) legend(off)	///
	plot(scatter petallen sepallen, msymbol(oh))	///				
	ylabel(0(10)70) xlabel(40(10)80)		///
	saving(fig12c.gph, replace)                                /* 12c */
assert reldif( 5.52166, r(min_y)) < 1e-4
assert reldif( 69.63744, r(max_y)) < 1e-4
assert reldif( 43.3959, r(min_x)) < 1e-4
assert reldif( 73.4713, r(max_x)) < 1e-4

ellip petallen sepalwid, c(pt2) l(80) legend(off)	///
	plot(scatter petallen sepalwid, msymbol(oh))	///				
	ylabel(0(10)70) xlabel(20(10)40)		///
	saving(fig12d.gph, replace)                                /* 12d */
assert reldif( 5.5222, r(min_y)) < 1e-4
assert reldif( 69.6383, r(max_y)) < 1e-4
assert reldif( 22.6581, r(min_x)) < 1e-4
assert reldif( 38.4888, r(max_x)) < 1e-4


* --------------------------- Example 13 (means) ----------------------
// Stata (2001, 4) [R] hadimvo -- Figure
// Stata 8 has removed manual entry for [R] hadimvo
* Cf. contour plots, for example Cook and Weisberg (1999, 64-68)
use hadimvo, clear

program define chminmax
	assert reldif( 3.8696, r(min_y)) < 1e-4
	assert reldif( 6.1546, r(max_y)) < 1e-4
	assert reldif( 3.7283, r(min_x)) < 1e-4
	assert reldif( 4.8916, r(max_x)) < 1e-4
end

// test the means default -- hardcode reasonable values
ellip y x, means nograph
chminmax

// test the overall default
ellip y x, nograph
chminmax

// the default is c(sd 2); Cramer (1946, 283-285) and McCartin (2002)
ellip y x, means c(sd 2) nograph
chminmax

// c(sd 2) = c(4)
ellip y x, means c(4) nograph
chminmax

// cf. test c(sd 4) in -hotelling-
ellip y x if outlier == 0, means c(sd 4) g(y2 x2) nograph /* -5 obs */
assert reldif( 2.89536, r(min_y)) < 1e-4
assert reldif( 6.95992, r(max_y)) < 1e-4
assert reldif( 3.90489, r(min_x)) < 1e-4
assert reldif( 4.89417, r(max_x)) < 1e-4
   
ellip y x, c(sd 4) legend(off)	///
	plot(line y2 x2 || scatter y x, msymbol(oh))	///
	ylabel(2(2)8) xlabel(3(1)6)	///
	saving(fig13a.gph, replace)                                /* 13a */
assert reldif( 2.72715, r(min_y)) < 1e-4
assert reldif( 7.29711, r(max_y)) < 1e-4
assert reldif( 3.14674, r(min_x)) < 1e-4
assert reldif( 5.47329, r(max_x)) < 1e-4	        

// Stata has lost the data and the code for the figure:
// 1. My smaller ellipse is slightly shorter than in the manual, which
//    could result from a user-error in Stage, I speculate.
// 2. c(sd 4) corresponds to 95% INDIVIDUAL confidence intervals,
//    or 86% JOINT confidence intervals.

// by ...: cannot be used with -graph, saving()-. You need a do-file:
use hadimvo, clear
local i = 1
egen grp = group(outlier), missing
while `i' <= 2 { 
	ellip y x if grp == `i', saving(fig13_`i', replace)
	local i = `i' + 1
}
graph use "fig13_1.gph" 
graph use "fig13_2.gph"

* test by()
use hadimvo, clear 
ellip y x, xlab(minmax) c(t2 2)
ellip y x, by(outlier) xlab(minmax) c(f 2)
ellip y x, by(outlier) xlab(minmax) c(sd 3)

// the following command failed before executable update 10apr2003
ellip y x, by(outlier) xlab(minmax) ///
	saving(fig13e.gph, replace)



* --------------------------- Example 14 (means) ----------------------
* Flury and Riedwyl (1988, 5-6, 16, 35-36, 141, 166-167) genuine Swiss
* bank notes data -- Figure 8.2, Figure 9.5, Figure 9.6
use notes, clear
qui summarize bottom
assert reldif( 8.305, r(mean)) < 1e-3
assert reldif( 7.2, r(min)) < 1e-2
assert reldif( 10.4, r(max)) < 1e-3
assert reldif( .6428, r(sd)) < 1e-3
qui summarize top
assert reldif( 10.168, r(mean)) < 1e-4
assert reldif( 7.7, r(min)) < 1e-2
assert reldif( 11.7, r(max)) < 1e-3
assert reldif( .6490, r(sd)) < 1e-3
qui mat accum Cov = bottom top, noc means(m) dev
assert 100 == r(N)
mat input m2 = ( 8.305, 10.168 )
assert mreldif( m2, m) < 1e-4
mat S = Cov/(r(N)-1)
mat input S2 = ( /*
    */  .4132, -.2635 \ /*
    */ -.2635,  .4212   /*
    */ )
assert mreldif( S2, S) < 1e-3
mat P = corr(S)
scalar r = P[1,2]
assert reldif( -.6316, r) < 1e-3

ellip top bottom, c(sd 2.2) gen(y2 x2) nograph
assert reldif( 8.74024, r(min_y)) < 1e-4
assert reldif( 11.5958, r(max_y)) < 1e-4
assert reldif( 6.89086, r(min_x)) < 1e-4
assert reldif( 9.71919, r(max_x)) < 1e-4

ellip top bottom, c(sd 3.8) gen(y4 x4) nograph
assert reldif( 7.70187, r(min_y)) < 1e-4
assert reldif( 12.6342, r(max_y)) < 1e-4
assert reldif( 5.86239, r(min_x)) < 1e-4
assert reldif( 10.7477, r(max_x)) < 1e-4

ellip top bottom, c(sd 5) legend(off)		///
	plot(line y2 x2 || line y4 x4 ||	///
	scatter top bottom, msymbol(oh))	///
	ylabel(6(1)14) xlabel(4(1)14)		///
	ytick(6.5(1)13.5) xtick(4.5(1)13.5)	///
	ytitle("TOP") xtitle("BOTTOM")		///
	saving(fig14a.gph, replace)                                /* 14a */
assert reldif( 6.9231, r(min_y)) < 1e-4
assert reldif( 13.4129, r(max_y)) < 1e-4
assert reldif( 5.09104, r(min_x)) < 1e-4
assert reldif( 11.5191, r(max_x)) < 1e-4

use notes, clear
* univariate 99% ci
qui ci top, level(99)
assert reldif( r(lb), 9.997) < 1e-4
assert reldif( r(ub), 10.339) < 1e-4
qui ci bottom, level(99)
assert reldif( r(lb), 8.136) < 1e-4
assert reldif( r(ub), 8.474) < 1e-4

* Figure 9.6 on p. 167 with 99% univariate and 99% hotelling ellipse
* Another example with c(t2) and l(99) is Morrison (1967, 121-123).
// how to produce shaded rectangle?
ellip top bottom, l(99) c(t2) gen(y99 x99) /*
    */ ylabel(9.80(.1)10.50) xlabel(7.80(.1)8.80) /*
    */ ytick(9.80(.05)10.50) xtick(7.80(.05)8.80) /*
    */ l1title(" ") l2title("TOP") b2title("BOTTOM") /*
    */ xline(8.136 8.474) yline(9.997 10.339) /*
    */ saving(fig14b.gph, replace)                                /* 14b */
* c(t2) was referred to as "standard distance"
assert reldif( 9.9653, r(min_y)) < 1e-4
assert reldif( 10.371, r(max_y)) < 1e-4
assert reldif( 8.1042, r(min_x)) < 1e-4
assert reldif( 8.5058, r(max_x)) < 1e-4	
assert reldif( .312^2, r(c)) < 1e-3

* Figure 9.5 on p. 166 with 95% and 99% hotelling ellipses
ellip top bottom, c(t2) legend(off)				///
	plot(line y99 x99 || scatter top bottom, msymbol(oh))	///
	ylabel(7.5(.5)12) xlabel(7(.5)11)			///
	ytick(7.75(.5)11.75) xtick(7.25(.5)10.75)		///
	ytitle("TOP") xtitle("BOTTOM")				///
	saving(fig14c.gph, replace)                                /* 14c */
assert reldif( 10.0059, r(min_y)) < 1e-4
assert reldif( 10.3301, r(max_y)) < 1e-4
assert reldif( 8.14441, r(min_x)) < 1e-4
assert reldif( 8.46559, r(max_x)) < 1e-4
assert reldif( .250^2, r(c)) < 1e-3


* --------------------------- Example 15 (means) ----------------------
* SAS OnlineDoc: Version 8, GPA data, p. 279 -- Figure 18.7
* Data are included with SAS/INSIGHT software
use gpa, clear
* 80% prediction hotelling: c(pt2 2)
ellip satm gpa, l(80) c(pt2) legend(off)	///
	plot(scatter satm gpa, msymbol(oh))	///
	ylabel(200(200)800) xlabel(2(1)6)	///
	ytick(300(200)700) xtick(2.5(1)5.5)	///
	saving(fig15a.gph, replace)     			/* 15a */		
assert reldif( 439.01, r(min_y)) < 1e-4
assert reldif( 751.559, r(max_y)) < 1e-4
assert reldif( 3.22556, r(min_x)) < 1e-4
assert reldif( 6.04493, r(max_x)) < 1e-4

ellip satm hsm, l(80) c(pt2) legend(off)	///
	plot(scatter satm hsm, msymbol(oh))	///
	ylabel(200(200)800) xlabel(2(2)12)	///
	ytick(300(200)700) xtick(3(2)11)	///
	saving(fig15b.gph, replace)     			/* 15b */	
assert reldif( 439.012, r(min_y)) < 1e-4
assert reldif( 751.561, r(max_y)) < 1e-4
assert reldif( 5.3575, r(min_x)) < 1e-4
assert reldif( 11.2854, r(max_x)) < 1e-4

ellip satm hss, l(80) c(pt2) legend(off)	///
	plot(scatter satm hss, msymbol(oh))	///
	ylabel(200(200)800) xlabel(2(2)12)	///
	ytick(300(200)700) xtick(3(2)11)	///
	saving(fig15c.gph, replace)     			/* 15c */
assert reldif( 439.01, r(min_y)) < 1e-4
assert reldif( 751.56, r(max_y)) < 1e-4
assert reldif( 5.01517, r(min_x)) < 1e-4
assert reldif( 11.1635, r(max_x)) < 1e-4

ellip satv gpa, l(80) c(pt2) legend(off)	///
	plot(scatter satv gpa, msymbol(oh))	///
	ylabel(200(200)800) xlabel(2(1)6)	///
	ytick(300(200)700) xtick(2.5(1)5.5)	///
	saving(fig15d.gph, replace)    				/* 15d */
assert reldif( 337.043, r(min_y)) < 1e-4
assert reldif( 672.051, r(max_y)) < 1e-4
assert reldif( 3.22556, r(min_x)) < 1e-4
assert reldif( 6.04493, r(max_x)) < 1e-4

ellip satv hsm, l(80) c(pt2) legend(off)	///
	plot(scatter satv hsm, msymbol(oh))	///
	ylabel(200(200)800) xlabel(2(2)12)	///
	ytick(300(200)700) xtick(3(2)11)	///
	saving(fig15e.gph, replace)     			/* 15e */	
assert reldif( 337.043, r(min_y)) < 1e-4
assert reldif( 672.052, r(max_y)) < 1e-4
assert reldif( 5.3575, r(min_x)) < 1e-4
assert reldif( 11.2854, r(max_x)) < 1e-4

ellip satv hss, l(80) c(pt2) legend(off)	///
	plot(scatter satv hss, msymbol(oh))	///
	ylabel(200(200)800) xlabel(2(2)12)	///
	ytick(300(200)700) xtick(3(2)11)	///
	saving(fig15f.gph, replace)     			/* 15f */	
assert reldif( 337.047, r(min_y)) < 1e-4
assert reldif( 672.055, r(max_y)) < 1e-4
assert reldif( 5.01517, r(min_x)) < 1e-4
assert reldif( 11.1635, r(max_x)) < 1e-4


* --------------------------- Example 16 (means) ----------------------
* SAS OnlineDoc: Version 8, miningx data, p. 617 -- Figure 39.5
* Data are included with SAS/INSIGHT software
use miningx, clear
* 80% and 50% prediction hotelling

* cf. Prediction ellipse around regression coefficients with GIVEN
* predictor values. An example is Johnson and Wichern
* (2002, 376-377, 397-398) computer data -- Figure 7.5.

ellip driltime depth, l(50) c(pt2) g(y50 x50) nograph
assert reldif( 5.33676, r(min_y)) < 1e-4
assert reldif( 9.81174, r(max_y)) < 1e-4
assert reldif( 63.3518, r(min_x)) < 1e-4
assert reldif( 341.652, r(max_x)) < 1e-4

ellip driltime depth, l(80) c(pt2) legend(off)	///
	plot(line y50 x50 || 			///
	scatter driltime depth, msymbol(oh))	///
	ylabel(4(2)12) xlabel(0(200)400)	///
	ytick(5(2)11) xtick(100(200)300)	///
	saving(fig16.gph, replace)     				/* 16 */
assert reldif( 4.1446, r(min_y)) < 1e-4
assert reldif( 11.0039, r(max_y)) < 1e-4
assert reldif( -10.7874, r(min_x)) < 1e-4
assert reldif( 415.794, r(max_x)) < 1e-4
	
	

* --------------------------- Example 17 (means) ----------------------
* Pearson (1901, 569) data reproduced in York (1966, 1085) 
// Unnamed figure in Pearson (1901, 570)
use pearson, clear

ellip y x, evr(1) nograph
assert reldif( 3.82, r(mean_x)) < 1e-3
assert reldif( 3.70, r(mean_y)) < 1e-3
assert reldif( -.546, r(m1)) < 1e-3

tempname l2 l1 m2 m1 c
scalar `l2' = r(lambda2)
scalar `l1' = r(lambda1)
scalar `m2' = r(m2)
scalar `m1' = r(m1)
scalar `c' = r(c)

// C, the product of eigenvalues = determinant of correlation matrix 
local C = `l1' * `l2' * `c'
local D = (`l2'*(1 + (`m1')^2))

// amax_
local amax_x = r(mean_x) + sqrt((`C')/(`D'))
local amax_y = r(mean_y) + r(m1) * (`amax_x' - r(mean_x)) 

// amin_
local amin_x = r(mean_x) - sqrt((`C')/(`D'))			// -1.180
local amin_y = r(mean_y) + r(m1) * (`amin_x' - r(mean_x))	// 6.428

ellip y x, evr(0) nograph
assert reldif( 3.82, r(mean_x)) < 1e-3
assert reldif( 3.70, r(mean_y)) < 1e-3
assert reldif( -.566, r(m1)) < 1e-3

ellip y x, evr(999) nograph
assert reldif( 3.82, r(mean_x)) < 1e-3
assert reldif( 3.70, r(mean_y)) < 1e-3
assert reldif( -.540, r(m1)) < 1e-3

ellip y x, plot(scatteri 3.7 3.82, msymbol(o) || scatter y x) ///
	legend(off) ylabel(0(2)8) xlabel(-2(2)10)	///
	ytick(1(2)7) xtick(-1(2)9)	
assert reldif( 3.82, r(mean_x)) < 1e-3
assert reldif( 3.70, r(mean_y)) < 1e-3
assert reldif( -.546, r(m1)) < 1e-3

ellip y x, evr(5) nograph



* --------------------------- Example 18 (means) ----------------------
* Sokal and Rohlf (1995), crab data, p.571 -- Figure 15.7
* TO DO: graph the orth reg lines, calculate theta from their formula
* TO DO: How to calculate a-d directly in ellip, that is, when evr(1)?
use crab, clear
qui corr x y, cov
assert reldif( 42.04347, r(Var_1)) < 1e-4
assert reldif( 11306.26465, r(Var_2)) < 1e-4
assert reldif( 596.51067, r(cov_12)) < 1e-4

* 95% hotelling
ellip y x, c(t2) gen(ye xe) evr(1) nograph
assert reldif( 12.04750, r(mean_x)) < 1e-5
assert reldif( 195.58333, r(mean_y)) < 1e-5
assert reldif( 11337.76550, r(lambda1)) < 1e-5
assert reldif( 10.54261, r(lambda2)) < 1e-5
assert reldif( 18.93633, r(m1)) < 1e-5
assert reldif( -.0528, r(m2)) < 1e-4
assert reldif( 92.347, r(a)) < 1e-4
assert reldif( 2.8160, r(b)) < 1e-4
assert reldif( 6.42411, r(min_x)) < 1e-4
assert reldif( 17.6711, r(max_x)) < 1e-4
assert reldif( 103.364, r(min_y)) < 1e-4
assert reldif( 287.801, r(max_y)) < 1e-4
assert reldif( .86519, r(r)) < 1e-4
assert reldif( 195.4348, r(bmin_y)) < 1e-4
 
// generate marginal confidence interval (rectangular boundary)
tempvar y_minmax x_minmax y_min x_min y_max x_max
gen `y_minmax' = r(min_y)		//
qui replace `y_minmax' = r(max_y) in 2	// 

gen `x_minmax' = r(min_x)
qui replace  `x_minmax' = r(max_x) in 2

gen `y_min' =  r(min_y)
gen `x_min' =  r(min_x)
gen `y_max' =  r(max_y)
gen `x_max' =  r(max_x)
tempvar yvar xvar n
gen `yvar' = r(min_y)
qui replace `yvar' = r(max_y) in 2/3
gen `xvar' = r(min_x)
qui replace `xvar' = r(max_x) in 3/4
gen `n' = _n

// center point
local mean_y = r(mean_y)				// 195.583
local mean_x = r(mean_x)				// 12.047

// point a -- evr(infinite)
// local bimin_x = `mean_x'
// local bimax_y = r(bmax_y)				// cf. 241.811

// point b -- evr(infinite)
// local bimax_x = `mean_x'
// local bimin_y = r(bmin_y)				// cf. 149.356

// point c -- evr(0)
// local b0max_x = r(max_x)				// cf. 14.866
// local b0min_y = `mean_y'

// point d -- evr(0)
// local b0min_x = r(bmin_x)				// cf.  9.228
// local b0max_y = `mean_y'

// point e
local amax_y = r(amax_y)
local amax_x = r(amax_x)

// point f
local amin_y = r(amin_y)
local amin_x = r(amin_x)

// point g
local bmin_y = r(bmin_y)
local bmax_x = r(bmax_x)

// point h
local bmax_y = r(bmax_y)
local bmin_x = r(bmin_x)


// plot() is very flexible; cf. Figure 15.7
ellip y x, c(t2) evr(1)                                 /// plot 1
	plot(line `yvar' `xvar', sort(`n') ||	        /// plot 2
	scatter y x, ms(x) ||				/// plot 3
	scatteri `amax_y' `amax_x' "e", ms(oh) ||	/// plot 4
	scatteri `amin_y' `amin_x' "f", ms(oh) ||	/// plot 5
	scatteri `bmin_y' `bmax_x' "g", ms(oh)  ||	/// plot 6
	scatteri `bmax_y' `bmin_x' "h", ms(oh)  ||	/// plot 7
	scatteri `mean_y' `mean_x', ms(oh))		/// plot 8
	ysize(2.6) xsize(1.89) legend(off)

tempname l2 l1 m2 m1 n s2_x ydiff s2_y xdiff c D
scalar `l2' = r(lambda2)
scalar `l1' = r(lambda1)
scalar `m2' = r(m2)
scalar `m1' = r(m1)
scalar `c' = r(c)
scalar `D' = (`l2'*(1 + (`m1')^2))

tempvar n
gen `n' = 12

* verify l1 and l2 	
qui pca y x, covariance
assert reldif( `l2', r(lambda2)) < 1e-6
assert reldif( `l1', r(lambda1)) < 1e-6

// point a -- evr(infinite)
scalar `l2' = round( r(lambda2),.00001) // pp. 590-591 have rounding errors
local F = round( invfprob(2,10,.05),.01)
local T2 = 2.2 * `F'
local C = `l1' * `l2' * `T2' / `n'
assert reldif( 89846.45, `C') < 1e-6
// local C = `l1' * `l2' * `c'	// 89908.269
qui summarize x
scalar `s2_x' = r(Var)
scalar `ydiff' = sqrt((`C')/(`s2_x'))
local bimin_x = `mean_x'
local bimax_y = `mean_y' + `ydiff'			// 241.811

// point b -- evr(infinite)
local bimax_x = `mean_x'
local bimin_y = `mean_y' - `ydiff'			// 149.356

// point c -- evr(0)
qui summarize y
scalar `s2_y' = r(Var)
scalar `xdiff' = sqrt((`C')/(`s2_y'))
local b0max_x = `mean_x' + `xdiff'			//  14.866
local b0min_y = `mean_y'

// point d -- evr(0)
local b0min_x = `mean_x' - `xdiff'			//   9.228
local b0max_y = `mean_y'

// point e -- evr(1)
local amax_x = `mean_x' + sqrt((`C')/(`D'))		// 17.047
local amax_y = `mean_y' + `m1' * (`amax_x' - `mean_x')	// 290.268

// point f -- evr(1)
local amin_x = `mean_x' - sqrt((`C')/(`D'))		// 7.047
local amin_y = `mean_y' + `m1' * (`amin_x' - `mean_x')	// 100.898

// point g -- evr(1)
local E = (`l1'*(1 + (`m2')^2))
local bmax_x = `mean_x' + sqrt((`C')/(`E'))		//  14.858
local bmin_y = `mean_y' + `m2' * (`bmax_x' - `mean_x')	// 195.435

// point h -- evr(1)
local bmin_x = `mean_x' - sqrt((`C')/(`E'))		// 9.236
local bmax_y = `mean_y' + `m2' * (`bmin_x' - `mean_x')	// 195.7318

// plot() is very flexible; cf. Figure 15.7
ellip y x, c(t2) evr(1)                                 /// plot  1
	plot(line `yvar' `xvar', sort(`n') ||	        /// plot  2
	scatter y x, ms(x) ||				/// plot  3
	scatteri `bimin_y' `bimax_x' "a", ms(oh) ||	/// plot  4
	scatteri `bimax_y' `bimin_x' "b", ms(oh) ||	/// plot  5	
	scatteri `b0min_y' `b0max_x' "c", ms(Sh) ||	/// plot  6	
	scatteri `b0max_y' `b0min_x' "d", ms(Sh) ||	/// plot  7 
	scatteri `amax_y' `amax_x' "e", ms(oh) ||	/// plot  8
	scatteri `amin_y' `amin_x' "f", ms(oh) ||	/// plot  9
	scatteri `bmin_y' `bmax_x' "g", ms(oh)  ||	/// plot 10
	scatteri `bmax_y' `bmin_x' "h", ms(oh)  ||	/// plot 11
	scatteri `mean_y' `mean_x', ms(oh))		/// plot 12
	ysize(2.6) xsize(1.89) legend(off)
	
