* version 1.0.1  Anders Alexandersson 5jul2004 (minor cleanup)
* version 1.0.0  Anders Alexandersson 29apr2004

/*

Methods and Formulas not mentioned in Alexandersson (2004)
==========================================================

There are two versions of the parametric scalar equations of the
confidence ellipse. In the following, let x and y be two half-ellipse
or arc variables in the (X,Y) coordinate system, xbar and ybar be the
means for x and y, a be the major axis, b be the minor axis, z be the
rotation angle of the major axis, t be the variable angle
(0 <= t <= 2pi or 0 <= t <= 360 degrees), and r the linear correlation
coefficient. Batschelet (1981) and others mention the "theta" equations

        x = xbar + a * cos(z) * cos(t) - b * sin(z) * sin(t)        (1)
        y = ybar + a * sin(z) * cos(t) + b * cos(z) * sin(t)        (2)

and Douglas (1993) mentions the "acosr" equations

        x = xbar + x_a * cos(t)                                     (3)
        y = ybar + y_a * cos(t + acos r).                           (4)

ellip uses both formulas but defaults to the acosr formula, because
r is easier to calculate and explain than z. In Douglas' version, the
x and y amplitudes are s_x * sqrt(c), and s_y * sqrt(c), respectively,
where c is the boundary constant. In ellip for means, s_x and s_y are
the standard deviations for x and y. In ellip for regression
coefficients, s_x and s_y are the standard errors for xbar and ybar,
respectively.

The SJ article uses the concept of angular distance to justify the ellipse
formula. Here is a more formal way to derive the standardized formulas in
ellip, thanks to private correspondence from J. B. Douglas, the same
person as in Douglas (1993). The algebraic scalar equation of the
standardized confidence ellipse is,
           1
        ------- * (x^2 - 2rxy + y^2) = c,
        1 - r^2
that is,

           1
        ----------- * [(y - rx)^2 + x^2(1 - r^2)] = 1.
        c (1 - r^2)

Multiply out the [] bracket, and simplify:

             1                      x^2
        ------------ * (y - rx)^2 + ---- = 1,
        c (1 - r^2)                  c
 
Take the square root, and convert to parametric form:
           x
        ------- = cos t
        sqrt(c)
and
        sqrt(1)
        ------------------ * (y - rx) = sin t.
        sqrt[ c(1 - r^2) ]

Rearrange the equations in terms of x and y:
        x = sqrt(c) * cos t
and
	y = rx + sqrt[ c(1 - r^2) ] * sin t

Insert sqrt(c) * cos t instead of x, and break out sqrt(c):
        y = sqrt(c) * [r * cos t + sqrt(1 - r^2) * sin t]
	
Let r = cos z:
        y = sqrt(c) * [cos z * cos t + sqrt(1 - cos z^2) * sin t]

From the fundamental trigonometric identity, (cos t)^2 + (sin t)^2 = 1,  
insert (sin z)^2 instead of 1 - (cos z)^2:
	y = sqrt(c) * [cos z * cos t + sqrt(sin z)^2 * sin t]
	y = sqrt(c) * cos z * cos t + sin z * sin t              (or: -sin z)
        y = sqrt(c) * cos(t + z)
          = sqrt(c) * cos(t + acos r).

Hans Ehrbart provide a third, trigonometric, derivation in some of his
econometric classnotes on the web page www.econ.utah.edu/ehrbar/ecmet.htm
[accessed 8 February 2004]. For Chapter 12, go to "12.3.2 Level Lines of the
Normal Density", problem 165 and the answer (2 pages).

I did not find a use for calculating the angle of the normal on the opposite
side (where the angle is not 90 degrees). But the formula is at
http://mathworld.wolfram.com/EllipseTangent.html [accessed 8 February 2004].


References not mentioned in Alexandersson (2004)
================================================

Bartels, L. 1996. Pooling Disparate Observations. American Journal of
    Political Science 40: 905-942.   

Cook, D. and Weisberg, S. 1999. Applied Regression Including Computing
    and Graphics. New York: John Wiley and Sons.    

Draper, N., and H. Smith. 1998. 3rd ed. Applied Regression Analysis.
    New York: John Wiley & Sons.
    
Flury, B., and H. Riedwyl. 1988. Multivariate Statistics: A Practical
    Approach. London and New York: Chapman and Hall.
    
Friendly, M. 1991. SAS System for Statistical Graphs, First Edition.
    Cary, NC: SAS Institute Inc.        

Greene, W. 2000. 4th ed. Econometric Analysis. Englewood Cliffs, NJ:
    Prentice Hall. 
    
Hotelling, H. 1931. The generalization of Student's ratio. Annals of
	Mathematical Statistics 2(3): 360-378.      

Jobson, J. 1991. Applied Multivariate Data Analysis. Volume 1:
    Regression and Experimental Design. New York: Springer-Verlag.
    
Johnson, R., and D. Wichern. 1998. 4th ed. Applied Multivariate
    Statistical Analysis. Upper Saddle River, NJ: Prentice Hall.    

Johnston, J. 1984. 3rd ed. Econometric Methods. New York: McGraw-Hill.
    
Monette, G. 1990. Geometry of multiple regression and interactive 3-D graphics.
	in Modern Methods of Data Analysis, eds. J. Fox, J. S. Long. Sage
	209-256.     
	
Montgomery, D., Peck. E., and G. Vining. 2001. 3rd ed. Introduction to
    Linear Regression Analysis. New York: John Wiley & Sons.

Montgomery, D., and E. Peck. 1982. Introduction to Linear Regression
    Analysis. New York: John Wiley & Sons.

Morrison, D. 1967. Multivariate Statistical Methods. New York:
    McGraw-Hill.

Morrison, D. 1983. Applied Linear Statistical Methods. Englewood
    Cliffs, NJ: Prentice Hall.  
    
Paradowski, Leslaw. 1997. "Uncertainty Ellipses and Their Application
    to Interval Estimation of Emitter Position", IEEE Transactions on
    Aerospace and Electronic Systems 33(1): 126-133.
    
Penny, Kay I. 1996. "Appropriate Critical Values When Testing for a
    Single Multivariate Outlier by Using the Mahalanobis Distance",
    Applied Statistics 45(1): 157-161.         

Sokal, R. R. and F. J. Rohlf. 1995. 3rd ed. Biometry: The Principles
    and Practice of Statistics in Biological Research. New York: W. H.
    Freeman and Company.

Theil. H., Principles of Econometrics. 1971. New York: John Wiley &
    Sons.

Weisberg, S. 1985. 2nd ed. Applied Linear Regression. New York: John
    Wiley & Sons.
    
York, D. 1966. "Least-Squares Fitting of a Straight Line", Canadian
    Journal of Physics 44(4): 1079-1086.
    
*/

about

do coefs
do means
